#ifndef CYGONCE_HAL_MPC555_REGS_H
#define CYGONCE_HAL_MPC555_REGS_H

//--------------------------------------------------------------------------------
// Include files
//
#include <cyg/hal/immr.h>
#include <pkgconf/hal.h>

//--------------------------------------------------------------------------------
// SPR access macros
#define CYGARC_MTSPR(_spr_, _v_) \
  asm volatile ("mtspr %0, %1;" : : "I" (_spr_), "r" (_v_));
#define CYGARC_MFSPR(_spr_, _v_) \
  asm volatile ("mfspr %0, %1;" : "=r" (_v_) : "I" (_spr_));

//---------------------------------------------------------------------------------
// TB access macros
#define CYGARC_CLEARTB  \
  CYGARC_MTSPR(284, 0); \
  CYGARC_MTSPR(285, 0);

#define CYGARC_TBSETREFA(_v_) \
  TBREF0 = _v_;

#define CYGARC_TBSETREFB(_v_) \
  TBREF1 = _v_;

//---------------------------------------------------------------------------------
// Generic definitions
//---------------------------------------------------------------------------------
// Interrupt priorities
// Notice that high priority means low value and vice versa.
// The levels detected by the SIU interrupt controller
#define CYGARC_IRQ_PRIORITY_HIGH    0
#define CYGARC_IRQ_PRIORITY_LOW     7

// The levels detected by the IMB3 interrupt controller
// IMB3 priorities are mapped to SIU levels as follows
// priority 0:6  ==>> level 0:6
// priority 7:31 ==>> level 7
#define CYGARC_IMB3_PRIORITY_HIGH   0
#define CYGARC_IMB3_PRIORITY_LOW    31

// Some SPRs
#define CYGARC_REG_XER		1
#define CYGARC_REG_LR		8
#define CYGARC_REG_CTR		9
#define CYGARC_REG_DSISR	18
#define CYGARC_REG_DAR		19
#define CYGARC_REG_DEC		22
#define CYGARC_REG_SRR0		26
#define CYGARC_REG_SRR1		27
#define CYGARC_REG_EIE		80
#define CYGARC_REG_EID		81
#define CYGARC_REG_NRI		82
#define CYGARC_REG_CMPA		144
#define CYGARC_REG_CMPB		145
#define CYGARC_REG_CMPC		146
#define CYGARC_REG_CMPD		147
#define CYGARC_REG_ECR		148
#define CYGARC_REG_DER		149
#define CYGARC_REG_COUNTA	150
#define CYGARC_REG_COUNTB	151
#define CYGARC_REG_CMPE		152
#define CYGARC_REG_CMPF		153
#define CYGARC_REG_CMPG		154
#define CYGARC_REG_CMPH		155
#define CYGARC_REG_LCTRL1	156
#define CYGARC_REG_LCTRL2	157
#define CYGARC_REG_ICTRL	158
#define CYGARC_REG_BAR		159
#define CYGARC_REG_TBL_R	268
#define CYGARC_REG_TBU_R	269
#define CYGARC_REG_SPRG0	272
#define CYGARC_REG_SPRG1	273
#define CYGARC_REG_SPRG2	274
#define CYGARC_REG_SPRG3	275
#define CYGARC_REG_TBL_W	284
#define CYGARC_REG_TBU_W	285
#define CYGARC_REG_PVR		287
#define CYGARC_REG_MI_GRA	528
#define CYGARC_REG_L2U_GRA	536
#define CYGARC_REG_BBCMCR	560
#define CYGARC_REG_L2U_MCR	568
#define CYGARC_REG_DPDR		630
#define CYGARC_REG_IMMR		638
#define CYGARC_REG_MI_RBA0	784
#define CYGARC_REG_MI_RBA1	785
#define CYGARC_REG_MI_RBA2	786
#define CYGARC_REG_MI_RBA3	787
#define CYGARC_REG_L2U_RBA0	792
#define CYGARC_REG_L2U_RBA1	793
#define CYGARC_REG_L2U_RBA2	794
#define CYGARC_REG_L2U_RBA3	795
#define CYGARC_REG_MI_RA0	816
#define CYGARC_REG_MI_RA1	817
#define CYGARC_REG_MI_RA2	818
#define CYGARC_REG_MI_RA3	819
#define CYGARC_REG_L2U_RA0	824
#define CYGARC_REG_L2U_RA1	825
#define CYGARC_REG_L2U_RA2	826
#define CYGARC_REG_L2U_RA3	827
#define CYGARC_REG_FPECR	1022

#ifdef CYGARC_HAL_COMMON_EXPORT_CPU_MACROS
//#define XER      CYGARC_REG_XER	// Left out, conflicts with the GDB stub
//#define LR       CYGARC_REG_LR        // They are not used anyway ...
#define CTR      CYGARC_REG_CTR
#define DSISR    CYGARC_REG_DSISR
#define DAR      CYGARC_REG_DAR
#define DEC      CYGARC_REG_DEC
#define SRR0     CYGARC_REG_SRR0
#define SRR1     CYGARC_REG_SRR1
#define EIE      CYGARC_REG_EIE
#define EID      CYGARC_REG_EID
#define NRI      CYGARC_REG_NRI
#define CMPA     CYGARC_REG_CMPA
#define CMPB     CYGARC_REG_CMPB
#define CMPC     CYGARC_REG_CMPC
#define CMPD     CYGARC_REG_CMPD
#define ECR      CYGARC_REG_ECR
#define DER      CYGARC_REG_DER
#define COUNTA   CYGARC_REG_COUNTA
#define COUNTB   CYGARC_REG_COUNTB
#define CMPE     CYGARC_REG_CMPE
#define CMPF     CYGARC_REG_CMPF
#define CMPG     CYGARC_REG_CMPG
#define CMPH     CYGARC_REG_CMPH
#define LCTRL1   CYGARC_REG_LCTRL1
#define LCTRL2   CYGARC_REG_LCTRL2
#define ICTRL    CYGARC_REG_ICTRL
#define BAR      CYGARC_REG_BAR
#define TBL_R    CYGARC_REG_TBL_R
#define TBU_R    CYGARC_REG_TBU_R
#define SPRG0    CYGARC_REG_SPRG0
#define SPRG1    CYGARC_REG_SPRG1
#define SPRG2    CYGARC_REG_SPRG2
#define SPRG3    CYGARC_REG_SPRG3
#define TBL_W    CYGARC_REG_TBL_W
#define TBU_W    CYGARC_REG_TBU_W
#define PVR      CYGARC_REG_PVR
#define MI_GRA   CYGARC_REG_MI_GRA
#define L2U_GRA  CYGARC_REG_L2U_GRA
#define BBCMCR   CYGARC_REG_BBCMCR
#define L2U_MCR  CYGARC_REG_L2U_MCR
#define DPDR     CYGARC_REG_DPDR
#define IMMR     CYGARC_REG_IMMR
#define MI_RBA0  CYGARC_REG_MI_RBA0
#define MI_RBA1  CYGARC_REG_MI_RBA1
#define MI_RBA2  CYGARC_REG_MI_RBA2
#define MI_RBA3  CYGARC_REG_MI_RBA3
#define L2U_RBA0 CYGARC_REG_L2U_RBA0
#define L2U_RBA1 CYGARC_REG_L2U_RBA1
#define L2U_RBA2 CYGARC_REG_L2U_RBA2
#define L2U_RBA3 CYGARC_REG_L2U_RBA3
#define MI_RA0   CYGARC_REG_MI_RA0
#define MI_RA1   CYGARC_REG_MI_RA1
#define MI_RA2   CYGARC_REG_MI_RA2
#define MI_RA3   CYGARC_REG_MI_RA3
#define L2U_RA0  CYGARC_REG_L2U_RA0
#define L2U_RA1  CYGARC_REG_L2U_RA1
#define L2U_RA2  CYGARC_REG_L2U_RA2
#define L2U_RA3  CYGARC_REG_L2U_RA3
#define FPECR    CYGARC_REG_FPECR
#endif // #ifdef CYGARC_HAL_COMMON_EXPORT_CPU_MACROS

//-----------------------------------------------------------------------------
// Development support
#define CYGARC_REG_ICTRL_SERSHOW       0x00000000  // serialized, show cycles 
#define CYGARC_REG_ICTRL_NOSERSHOW     0x00000007  // non serialized, no show cycles

#ifdef CYGARC_HAL_COMMON_EXPORT_CPU_MACROS
#define ICTRL_SERSHOW     CYGARC_REG_ICTRL_SERSHOW  
#define ICTRL_NOSERSHOW   CYGARC_REG_ICTRL_NOSERSHOW
#endif

//-----------------------------------------------------------------------------
// The internal memory map register (IMMR)
#define CYGARC_REG_IMMR_PARTNUM        0xff000000  // part number mask (ro)
#define CYGARC_REG_IMMR_MASKNUM        0x00ff0000  // mask number mask (ro)
#define CYGARC_REG_IMMR_ISB            0x0000000e  // internal space base
#define CYGARC_REG_IMMR_CLES           0x00000100  // core little endian swap
#define CYGARC_REG_IMMR_FLEN           0x00000800  // flash enablei

//-----------------------------------------------------------------------------
// System protection control (SYPCR)
#define CYGARC_REG_IMM_SYPR_SWTC_MASK  0xffff0000
#define CYGARC_REG_IMM_SYPR_BMT_MASK   0x0000ff00
#define CYGARC_REG_IMM_SYPR_BME        0x00000080
#define CYGARC_REG_IMM_SYPR_SWF        0x00000008
#define CYGARC_REG_IMM_SYPR_SWE        0x00000004
#define CYGARC_REG_IMM_SYPR_SWRI       0x00000002 
#define CYGARC_REG_IMM_SYPR_SWP        0x00000001

//-----------------------------------------------------------------------------
// Interrupt pend register (SIPEND)
#define CYGARC_REG_IMM_SIPEND_IRQ0     0x80000000  // External interrupt priority 0
#define CYGARC_REG_IMM_SIPEND_LVL0     0x40000000  // Internal interrupt level 0
#define CYGARC_REG_IMM_SIPEND_IRQ1     0x20000000  // External interrupt priority 1
#define CYGARC_REG_IMM_SIPEND_LVL1     0x10000000  // Internal interrupt level 1
#define CYGARC_REG_IMM_SIPEND_IRQ2     0x08000000  // External interrupt priority 2
#define CYGARC_REG_IMM_SIPEND_LVL2     0x04000000  // Internal interrupt level 2
#define CYGARC_REG_IMM_SIPEND_IRQ3     0x02000000  // External interrupt priority 3
#define CYGARC_REG_IMM_SIPEND_LVL3     0x01000000  // Internal interrupt level 3
#define CYGARC_REG_IMM_SIPEND_IRQ4     0x00800000  // External interrupt prioeity 4
#define CYGARC_REG_IMM_SIPEND_LVL4     0x00400000  // Internal interrupt level 4
#define CYGARC_REG_IMM_SIPEND_IRQ5     0x00200000  // External interrupt priority 5
#define CYGARC_REG_IMM_SIPEND_LVL5     0x00100000  // Internal interrupt level 5
#define CYGARC_REG_IMM_SIPEND_IRQ6     0x00080000  // External interrupt priority 6
#define CYGARC_REG_IMM_SIPEND_LVL6     0x00040000  // Internal interrupt level 6
#define CYGARC_REG_IMM_SIPEND_IRQ7     0x00020000  // External interrupt priority 7
#define CYGARC_REG_IMM_SIPEND_LVL7     0x00010000  // Internal interrupt level 7

//-----------------------------------------------------------------------------
// Interrupt mask register (SIMASK)
#define CYGARC_REG_IMM_SIMASK_IRM0     0x80000000  // External interrupt priority 0
#define CYGARC_REG_IMM_SIMASK_LVM0     0x40000000  // Internal interrupt level 0
#define CYGARC_REG_IMM_SIMASK_IRM1     0x20000000  // External interrupt priority 1
#define CYGARC_REG_IMM_SIMASK_LVM1     0x10000000  // Internal interrupt level 1
#define CYGARC_REG_IMM_SIMASK_IRM2     0x08000000  // External interrupt priority 2
#define CYGARC_REG_IMM_SIMASK_LVM2     0x04000000  // Internal interrupt level 2
#define CYGARC_REG_IMM_SIMASK_IRM3     0x02000000  // External interrupt priority 3
#define CYGARC_REG_IMM_SIMASK_LVM3     0x01000000  // Internal interrupt level 3
#define CYGARC_REG_IMM_SIMASK_IRM4     0x00800000  // External interrupt priority 4
#define CYGARC_REG_IMM_SIMASK_LVM4     0x00400000  // Internal interrupt level 4
#define CYGARC_REG_IMM_SIMASK_IRM5     0x00200000  // External interrupt priority 5
#define CYGARC_REG_IMM_SIMASK_LVM5     0x00100000  // Internal interrupt level 5
#define CYGARC_REG_IMM_SIMASK_IRM6     0x00080000  // External interrupt priority 6
#define CYGARC_REG_IMM_SIMASK_LVM6     0x00040000  // Internal interrupt level 6
#define CYGARC_REG_IMM_SIMASK_IRM7     0x00020000  // External interrupt priority 7
#define CYGARC_REG_IMM_SIMASK_LVM7     0x00010000  // Internal interrupt level 7

//-----------------------------------------------------------------------------
// Interrupt edge level register (CIEL)
#define CYGARC_REG_IMM_SIEL_ED0        0x80000000  // Falling edge external interrupt priority 0
#define CYGARC_REG_IMM_SIEL_WM0        0x40000000  // Wake-up mask external interrupt priority 0
#define CYGARC_REG_IMM_SIEL_ED1        0x20000000  // Falling edge external interrupt priority 1
#define CYGARC_REG_IMM_SIEL_WM1        0x10000000  // Wake-up mask external interrupt priority 1
#define CYGARC_REG_IMM_SIEL_ED2        0x08000000  // Falling edge external interrupt priority 2
#define CYGARC_REG_IMM_SIEL_WM2        0x04000000  // Wake-up mask external interrupt priority 2
#define CYGARC_REG_IMM_SIEL_ED3        0x02000000  // Falling edge external interrupt priority 3
#define CYGARC_REG_IMM_SIEL_WM3        0x01000000  // Wake-up mask external interrupt priority 3
#define CYGARC_REG_IMM_SIEL_ED4        0x00800000  // Falling edge external interrupt prioriry 4
#define CYGARC_REG_IMM_SIEL_WM4        0x00400000  // Wake-up mask external interrupt priority 4
#define CYGARC_REG_IMM_SIEL_ED5        0x00200000  // Falling edge external interrupt priority 5
#define CYGARC_REG_IMM_SIEL_WM5        0x00100000  // Wake-up mask external interrupt priority 5
#define CYGARC_REG_IMM_SIEL_ED6        0x00080000  // Falling edge external interrupt priority 6
#define CYGARC_REG_IMM_SIEL_WM6        0x00040000  // Wake-up mask external interrupt priority 6
#define CYGARC_REG_IMM_SIEL_ED7        0x00020000  // Falling edge external interrupt priority 7
#define CYGARC_REG_IMM_SIEL_WM7        0x00010000  // Wake-up mask external interrupt priority 7

//-----------------------------------------------------------------------------
// Memory controller
#define CYGARC_REG_IMM_BR_BA_MASK      0xffff8000       // base address
#define CYGARC_REG_IMM_BR_AT_MASK      0x00007000       // address type
#define CYGARC_REG_IMM_BR_PS_8         0x00000400       // port size 8 bits
#define CYGARC_REG_IMM_BR_PS_16        0x00000800       // port size 16 bits
#define CYGARC_REG_IMM_BR_PS_32        0x00000000       // port size 32 bits
#define CYGARC_REG_IMM_BR_WP           0x00000100       // write protect
#define CYGARC_REG_IMM_BR_WEBS         0x00000020       // write enable byte select
#define CYGARC_REG_IMM_BR_TBDIP        0x00000010       // toggle burst data in progress
#define CYGARC_REG_IMM_BR_LBDIP        0x00000008       // late burst data in progress
#define CYGARC_REG_IMM_BR_SETA         0x00000004       // externam transfer acknowledge
#define CYGARC_REG_IMM_BR_BI           0x00000002       // burst inhibit
#define CYGARC_REG_IMM_BR_V            0x00000001       // valid bit

#define CYGARC_REG_IMM_OR_AM           0xffff8000       // address mask   
#define CYGARC_REG_IMM_OR_ATM          0x00007000       // address type mask
#define CYGARC_REG_IMM_OR_CSNT         0x00000800       // GPCM : chip select negatoion time
#define CYGARC_REG_IMM_OR_ACS_0        0x00000000       // GPCM : CS output immediately
#define CYGARC_REG_IMM_OR_ACS_4        0x00000400       // GPCM : CS output 1/4 clock cycle later
#define CYGARC_REG_IMM_OR_ACS_2        0x00000600       // GPCM : CS output 1/2 clock cycle later
#define CYGARC_REG_IMM_OR_SCY_MASK     0x000000f0       // cycle length in clocks
#define CYGARC_REG_IMM_OR_BSCY         0x0000000E       // burst cycle length in clocks
#define CYGARC_REG_IMM_OR_TRLX         0x00000001       // timing relaxed
#define CYGARC_REG_IMM_OR_EHTR         0x00000100       // extended hold time on read
#define CYGARC_REG_IMM_OR_SCY_SHIFT    4            

//-----------------------------------------------------------------------------
// Time base status and control (TBSCR)
#define CYGARC_REG_IMM_TBSCR_REFA      0x0080           // reference interrupt status A
#define CYGARC_REG_IMM_TBSCR_REFB      0x0040           // reference interrupt status B
#define CYGARC_REG_IMM_TBSCR_REFAE     0x0008           // reference interrupt enable A
#define CYGARC_REG_IMM_TBSCR_REFBE     0x0004           // reference interrupt enable B
#define CYGARC_REG_IMM_TBSCR_TBF       0x0002           // timebase freeze
#define CYGARC_REG_IMM_TBSCR_TBE       0x0001           // timebase enable
#define CYGARC_REG_IMM_TBSCR_IRQ0      0x8000           // highest interrupt level
#define CYGARC_REG_IMM_TBSCR_IRQMASK   0xff00           // irq priority mask

//-----------------------------------------------------------------------------
// Real time clock
#define CYGARC_REG_IMM_RTCSC_SEC       0x0080           // once per second interrupt
#define CYGARC_REG_IMM_RTCSC_ALR       0x0040           // alarm interrupt
#define CYGARC_REG_IMM_RTCSC_4M        0x0010           // source select
#define CYGARC_REG_IMM_RTCSC_SIE       0x0008           // second interrupt enable
#define CYGARC_REG_IMM_RTCSC_ALE       0x0004           // alarm interrupt enable
#define CYGARC_REG_IMM_RTCSC_RTF       0x0002           // real time clock freeze
#define CYGARC_REG_IMM_RTCSC_RTE       0x0001           // real time clock enable
#define CYGARC_REG_IMM_RTCSC_IRQ0      0x8000           // highest interrupt priority
#define CYGARC_REG_IMM_RTCSC_IRQMASK   0xff00           // irq priority mask

//-----------------------------------------------------------------------------
// Periodic interrupt status an control
#define CYGARC_REG_IMM_PISCR_PS        0x0080           // periodic interrupt status
#define CYGARC_REG_IMM_PISCR_PIE       0x0004           // periodic interrupt enable
#define CYGARC_REG_IMM_PISCR_PITF      0x0002           // periodic interrupt timer freeze
#define CYGARC_REG_IMM_PISCR_PTE       0x0001           // periodic timer enable
#define CYGARC_REG_IMM_PISCR_IRQ0      0x8000           // highest intetrupt priority
#define CYGARC_REG_IMM_PISCR_IRQMASK   0xff00           // irq priority mask

//-----------------------------------------------------------------------------
// Queued analog to digital convertor
#define CYGARC_REG_IMM_QUACR1_CIE1       0x8000
#define CYGARC_REG_IMM_QUACR1_PIE1       0x4000
#define CYGARC_REG_IMM_QUACR1_SSE1       0x2000
#define CYGARC_REG_IMM_QUACR1_MO1        0x1f00
#define CYGARC_REG_IMM_QUACR2_CIE2       0x8000
#define CYGARC_REG_IMM_QUACR2_PIE2       0x4000
#define CYGARC_REG_IMM_QUACR2_SSE2       0x2000
#define CYGARC_REG_IMM_QUACR2_MO2        0x1f00
#define CYGARC_REG_IMM_QUACR2_RESUME     0x0080
#define CYGARC_REG_IMM_QUACR2_BQ2        0x007f
#define CYGARC_REG_IMM_QUASR0_CF1        0x8000
#define CYGARC_REG_IMM_QUASR0_PF1        0x4000
#define CYGARC_REG_IMM_QUASR0_CF2        0x2000
#define CYGARC_REG_IMM_QUASR0_PF2        0x1000
#define CYGARC_REG_IMM_QUASR0_TOR1       0x0800
#define CYGARC_REG_IMM_QUASR0_TOR2       0x0400
#define CYGARC_REG_IMM_QUASR0_QS         0x03c0
#define CYGARC_REG_IMM_QUASR0_CWP        0x003f
#define CYGARC_REG_IMM_QUADC64INT_IRL1   0xf800
#define CYGARC_REG_IMM_QUADC64INT_IRL2   0x07c0
#define CYGARC_REG_IMM_QUADC64INT_IRL1_SHIFT 11
#define CYGARC_REG_IMM_QUADC64INT_IRL2_SHIFT  6

//-----------------------------------------------------------------------------
// PLL Change of lock
#define CYGARC_REG_IMM_COLIR_IRQ0      0x8000           // the highest interrupt priority
#define CYGARC_REG_IMM_COLIR_COLIRQ    0xff00           // interrupt priority mask
#define CYGARC_REG_IMM_COLIR_COLIS     0x0080           // change of lock detected
#define CYGARC_REG_IMM_COLIR_COLIE     0x0040           // change of lock interrupt enable

//-----------------------------------------------------------------------------
// SCI SPI registers
#define CYGARC_REG_IMM_SCCxR1_LOOPS     0x4000
#define CYGARC_REG_IMM_SCCxR1_WOMS      0x2000
#define CYGARC_REG_IMM_SCCxR1_ILT       0x1000
#define CYGARC_REG_IMM_SCCxR1_PT        0x0800
#define CYGARC_REG_IMM_SCCxR1_PE        0x0400
#define CYGARC_REG_IMM_SCCxR1_M         0x0200
#define CYGARC_REG_IMM_SCCxR1_WAKE      0x0100
#define CYGARC_REG_IMM_SCCxR1_TIE       0x0080
#define CYGARC_REG_IMM_SCCxR1_TCIE      0x0040
#define CYGARC_REG_IMM_SCCxR1_RIE       0x0020
#define CYGARC_REG_IMM_SCCxR1_ILIE      0x0010
#define CYGARC_REG_IMM_SCCxR1_TE        0x0008
#define CYGARC_REG_IMM_SCCxR1_RE        0x0004
#define CYGARC_REG_IMM_SCCxR1_RWU       0x0002
#define CYGARC_REG_IMM_SCCxR1_SBK       0x0001
#define CYGARC_REG_IMM_QSCI1CR_QTPNT    0xf000
#define CYGARC_REG_IMM_QSCI1CR_QTHFI    0x0800
#define CYGARC_REG_IMM_QSCI1CR_QBHFI    0x0400
#define CYGARC_REG_IMM_QSCI1CR_QTHEI    0x0200
#define CYGARC_REG_IMM_QSCI1CR_QBHEI    0x0100
#define CYGARC_REG_IMM_QSCI1CR_QTE      0x0040
#define CYGARC_REG_IMM_QSCI1CR_QRE      0x0020
#define CYGARC_REG_IMM_QSCI1CR_QTWE     0x0010
#define CYGARC_REG_IMM_QSCI1CR_QTSZ     0x000f
#define CYGARC_REG_IMM_SPCR1_SPE        0x8000
#define CYGARC_REG_IMM_SPCR1_DSCLK      0x7f00  
#define CYGARC_REG_IMM_SPCR1_DTL        0x00ff
#define CYGARC_REG_IMM_SPCR2_SPIFIE     0x8000
#define CYGARC_REG_IMM_SPCR2_WREN       0x4000
#define CYGARC_REG_IMM_SPCR2_WRT0       0x2000
#define CYGARC_REG_IMM_SPCR2_ENDQP      0x1f00
#define CYGARC_REG_IMM_SPCR2_NEWQP      0x001f
#define CYGARC_REG_IMM_SPCR3_LOOPQ      0x0400
#define CYGARC_REG_IMM_SPCR3_HMIE       0x0200
#define CYGARC_REG_IMM_SPCR3_HALT       0x0100
#define CYGARC_REG_IMM_SPCR3_SPSR       0x00ff
#define CYGARC_REG_IMM_SCxSR_TDRE       0x0100
#define CYGARC_REG_IMM_SCxSR_TC         0x0080
#define CYGARC_REG_IMM_SCxSR_RDRF       0x0040
#define CYGARC_REG_IMM_SCxSR_RAF        0x0020
#define CYGARC_REG_IMM_SCxSR_IDLE       0x0010
#define CYGARC_REG_IMM_SCxSR_OR         0x0008
#define CYGARC_REG_IMM_SCxSR_NF         0x0004
#define CYGARC_REG_IMM_SCxSR_FE         0x0002
#define CYGARC_REG_IMM_SCxSR_PF         0x0001
#define CYGARC_REG_IMM_QSCI1SR_QOR      0x1000
#define CYGARC_REG_IMM_QSCI1SR_QTHF     0x0800
#define CYGARC_REG_IMM_QSCI1SR_QBHF     0x0400
#define CYGARC_REG_IMM_QSCI1SR_QTHE     0x0200
#define CYGARC_REG_IMM_QSCI1SR_QBHE     0x0100
#define CYGARC_REG_IMM_QSCI1SR_QRPNT    0x00f0
#define CYGARC_REG_IMM_QSCI1SR_QRPEND   0x000f
#define CYGARC_REG_IMM_SPSR_SPCR3       0xff00
#define CYGARC_REG_IMM_SPSR_SPIF        0x0080
#define CYGARC_REG_IMM_SPSR_MODF        0x0040
#define CYGARC_REG_IMM_SPSR_HALTA       0x0020
#define CYGARC_REG_IMM_SPSR_CPTQ        0x001f
#define CYGARC_REG_IMM_QDSCI_IL_ILDSCI  0x1f00
#define CYGARC_REG_IMM_QDSCI_IL_ILDSCI_SHIFT 8
#define CYGARC_REG_IMM_QSPI_IL_ILQSPI   0x001f
#define CYGARC_REG_IMM_QSPI_IL_ILQSPI_SHIFT  0

//-----------------------------------------------------------------------------
// TPU register settings
#define CYGARC_REG_IMM_TICR_CIRL       0x0700
#define CYGARC_REG_IMM_TICR_CIRL_SHIFT      8
#define CYGARC_REG_IMM_TICR_ILBS       0x00c0
#define CYGARC_REG_IMM_TICR_ILBS_SHIFT      6

//-----------------------------------------------------------------------------
// TOUCAN registers
#define CYGARC_REG_IMM_CANCTRL0_BOFFMSK  0x8000
#define CYGARC_REG_IMM_CANCTRL0_ERRMSK   0x4000
#define CYGARC_REG_IMM_CANCTRL0_RXMOD    0x0c00
#define CYGARC_REG_IMM_CANCTRL0_TXMOD    0x0300
#define CYGARC_REG_IMM_CANCTRL0_CANCTLL1 0x00ff
#define CYGARC_REG_IMM_TCNMCR_STOP       0x8000
#define CYGARC_REG_IMM_TCNMCR_FRZ        0x4000
#define CYGARC_REG_IMM_TCNMCR_HALT       0x1000
#define CYGARC_REG_IMM_TCNMCR_NOTRDY     0x0800
#define CYGARC_REG_IMM_TCNMCR_WAKEMSK    0x0400
#define CYGARC_REG_IMM_TCNMCR_SOFTRST    0x0200
#define CYGARC_REG_IMM_TCNMCR_FRZACK     0x0100
#define CYGARC_REG_IMM_TCNMCR_SUPV       0x0080
#define CYGARC_REG_IMM_TCNMCR_SELFWAKE   0x0040
#define CYGARC_REG_IMM_TCNMCR_APS        0x0020
#define CYGARC_REG_IMM_TCNMCR_STOPACK    0x0010
#define CYGARC_REG_IMM_ESTAT_BOFFINT     0x0004
#define CYGARC_REG_IMM_ESTAT_ERRINT      0x0002
#define CYGARC_REG_IMM_ESTAT_WAKEINT     0x0001
#define CYGARC_REG_IMM_CANICR_IRL        0x0700
#define CYGARC_REG_IMM_CANICR_IRL_SHIFT       8
#define CYGARC_REG_IMM_CANICR_ILBS       0x00c0
#define CYGARC_REG_IMM_CANICR_ILBS_SHIFT      6

//-----------------------------------------------------------------------------
// MIOS registers
#define CYGARC_REG_IMM_MIOS1ER0_EN15     0x8000
#define CYGARC_REG_IMM_MIOS1ER0_EN14     0x4000
#define CYGARC_REG_IMM_MIOS1ER0_EN13     0x2000
#define CYGARC_REG_IMM_MIOS1ER0_EN12     0x1000
#define CYGARC_REG_IMM_MIOS1ER0_EN11     0x0800
#define CYGARC_REG_IMM_MIOS1ER0_EN6      0x0040
#define CYGARC_REG_IMM_MIOS1ER0_EN3      0x0008
#define CYGARC_REG_IMM_MIOS1ER0_EN2      0x0004
#define CYGARC_REG_IMM_MIOS1ER0_EN1      0x0002
#define CYGARC_REG_IMM_MIOS1ER0_EN0      0x0001
#define CYGARC_REG_IMM_MIOS1ER1_EN31     0x8000
#define CYGARC_REG_IMM_MIOS1ER1_EN30     0x4000
#define CYGARC_REG_IMM_MIOS1ER1_EN29     0x2000
#define CYGARC_REG_IMM_MIOS1ER1_EN28     0x1000
#define CYGARC_REG_IMM_MIOS1ER1_EN27     0x0800
#define CYGARC_REG_IMM_MIOS1ER1_EN22     0x0040
#define CYGARC_REG_IMM_MIOS1ER1_EN19     0x0008
#define CYGARC_REG_IMM_MIOS1ER1_EN18     0x0004
#define CYGARC_REG_IMM_MIOS1ER1_EN17     0x0002
#define CYGARC_REG_IMM_MIOS1ER1_EN16     0x0001
#define CYGARC_REG_IMM_MIOS1SR0_FL15     0x8000
#define CYGARC_REG_IMM_MIOS1SR0_FL14     0x4000
#define CYGARC_REG_IMM_MIOS1SR0_FL13     0x2000
#define CYGARC_REG_IMM_MIOS1SR0_FL12     0x1000
#define CYGARC_REG_IMM_MIOS1SR0_FL11     0x0800
#define CYGARC_REG_IMM_MIOS1SR0_FL6      0x0040
#define CYGARC_REG_IMM_MIOS1SR0_FL3      0x0008
#define CYGARC_REG_IMM_MIOS1SR0_FL2      0x0004
#define CYGARC_REG_IMM_MIOS1SR0_FL1      0x0002
#define CYGARC_REG_IMM_MIOS1SR0_FL0      0x0001
#define CYGARC_REG_IMM_MIOS1SR1_FL31     0x8000
#define CYGARC_REG_IMM_MIOS1SR1_FL30     0x4000
#define CYGARC_REG_IMM_MIOS1SR1_FL29     0x2000
#define CYGARC_REG_IMM_MIOS1SR1_FL28     0x1000
#define CYGARC_REG_IMM_MIOS1SR1_FL27     0x0800
#define CYGARC_REG_IMM_MIOS1SR1_FL22     0x0040
#define CYGARC_REG_IMM_MIOS1SR1_FL19     0x0008
#define CYGARC_REG_IMM_MIOS1SR1_FL18     0x0004
#define CYGARC_REG_IMM_MIOS1SR1_FL17     0x0002
#define CYGARC_REG_IMM_MIOS1SR1_FL16     0x0001
#define CYGARC_REG_IMM_MIOS1LVL_LVL      0x0700
#define CYGARC_REG_IMM_MIOS1LVL_LVL_SHIFT     8
#define CYGARC_REG_IMM_MIOS1LVL_TM       0x00c0
#define CYGARC_REG_IMM_MIOS1LVL_TM_SHIFT      6

//-----------------------------------------------------------------------------
// Periodic interrupt timer count
#define CYGARC_REG_IMM_PITC_COUNT_SHIFT 16              // count is stored in bits 0-15

//-----------------------------------------------------------------------------
// System clock control   
#define CYGARC_REG_IMM_SCCR_TBS         0x02000000      // Time base source
#define CYGARC_REG_IMM_SCCR_RTDIV       0x01000000      // rtc clock divide
#define CYGARC_REG_IMM_SCCR_RTSEL       0x00100000      // rtc clock select

//-----------------------------------------------------------------------------
// MSR bits
#ifdef CYGARC_HAL_COMMON_EXPORT_CPU_MACROS
#define MSR_LE                         0x00000001	// little-endian mode enable
#define MSR_RI                         0x00000002	// recoverable exception
#define MSR_DR                         0x00000010	// data address translation
#define MSR_IR                         0x00000020	// instruction address translation
#define MSR_IP                         0x00000040	// exception prefix
#define MSR_FE1                        0x00000100	// floating point exception mode 1
#define MSR_BE                         0x00000200	// branche trace enable
#define MSR_SE                         0x00000400	// single-step trace enable
#define MSR_FE0                        0x00000800	// floating point exception mode 0
#define MSR_ME                         0x00001000	// machine check enable
#define MSR_FP                         0x00002000	// floating point available
#define MSR_PR                         0x00004000	// privilege level
#define MSR_EE                         0x00008000	// external interrupt enable
#define MSR_ILE                        0x00010000	// exception little-endian mode
#define MSR_POW                        0x00040000	// power management enable
#endif // #ifdef CYGARC_COMMON_EXPORT_CPU_MACROS

#define CYGARC_REG_IMM_BASE		IMMR_BASE
			
                        // General register definitions
#define CYGARC_REG_IMM_SIUMCR                      (CYGARC_REG_IMM_BASE + IMMR_SIUMCR)
#define CYGARC_REG_IMM_SYPCR                       (CYGARC_REG_IMM_BASE + IMMR_SYPCR)
#define CYGARC_REG_IMM_SWSR                        (CYGARC_REG_IMM_BASE + IMMR_SWSR)
#define CYGARC_REG_IMM_SIPEND                      (CYGARC_REG_IMM_BASE + IMMR_SIPEND)
#define CYGARC_REG_IMM_SIMASK                      (CYGARC_REG_IMM_BASE + IMMR_SIMASK)
#define CYGARC_REG_IMM_SIEL                        (CYGARC_REG_IMM_BASE + IMMR_SIEL)
#define CYGARC_REG_IMM_SIVEC                       (CYGARC_REG_IMM_BASE + IMMR_SIVEC)
#define CYGARC_REG_IMM_TESR                        (CYGARC_REG_IMM_BASE + IMMR_TESR)
#define CYGARC_REG_IMM_SGPIODT1                    (CYGARC_REG_IMM_BASE + IMMR_SGPIODT1)
#define CYGARC_REG_IMM_SGPIODT2                    (CYGARC_REG_IMM_BASE + IMMR_SGPIODT2)
#define CYGARC_REG_IMM_SGPIOCR                     (CYGARC_REG_IMM_BASE + IMMR_SGPIOCR)
#define CYGARC_REG_IMM_EMCR                        (CYGARC_REG_IMM_BASE + IMMR_EMCR)
#define CYGARC_REG_IMM_PDMCR                       (CYGARC_REG_IMM_BASE + IMMR_PDMCR)

			// Memory controller registers
#define CYGARC_REG_IMM_BR0                         (CYGARC_REG_IMM_BASE + IMMR_BR0)
#define CYGARC_REG_IMM_OR0                         (CYGARC_REG_IMM_BASE + IMMR_OR0)
#define CYGARC_REG_IMM_BR1                         (CYGARC_REG_IMM_BASE + IMMR_BR1)
#define CYGARC_REG_IMM_OR1                         (CYGARC_REG_IMM_BASE + IMMR_OR1)
#define CYGARC_REG_IMM_BR2                         (CYGARC_REG_IMM_BASE + IMMR_BR2)
#define CYGARC_REG_IMM_OR2                         (CYGARC_REG_IMM_BASE + IMMR_OR2)
#define CYGARC_REG_IMM_BR3                         (CYGARC_REG_IMM_BASE + IMMR_BR3)
#define CYGARC_REG_IMM_OR3                         (CYGARC_REG_IMM_BASE + IMMR_OR3)
#define CYGARC_REG_IMM_DMBR                        (CYGARC_REG_IMM_BASE + IMMR_DMBR)
#define CYGARC_REG_IMM_DMOR                        (CYGARC_REG_IMM_BASE + IMMR_DMOR)
#define CYGARC_REG_IMM_MSTAT                       (CYGARC_REG_IMM_BASE + IMMR_MSTAT)

			// System integration timers
#define CYGARC_REG_IMM_TBSCR                       (CYGARC_REG_IMM_BASE + IMMR_TBSCR)
#define CYGARC_REG_IMM_TBREF0                      (CYGARC_REG_IMM_BASE + IMMR_TBREF0)
#define CYGARC_REG_IMM_TBREF1                      (CYGARC_REG_IMM_BASE + IMMR_TBREF1)
#define CYGARC_REG_IMM_RTCSC                       (CYGARC_REG_IMM_BASE + IMMR_RTCSC)
#define CYGARC_REG_IMM_RTC                         (CYGARC_REG_IMM_BASE + IMMR_RTC)
#define CYGARC_REG_IMM_RTSEC                       (CYGARC_REG_IMM_BASE + IMMR_RTSEC)
#define CYGARC_REG_IMM_RTCAL                       (CYGARC_REG_IMM_BASE + IMMR_RTCAL)
#define CYGARC_REG_IMM_PISCR                       (CYGARC_REG_IMM_BASE + IMMR_PISCR)
#define CYGARC_REG_IMM_PITC                        (CYGARC_REG_IMM_BASE + IMMR_PITC)
#define CYGARC_REG_IMM_PITR                        (CYGARC_REG_IMM_BASE + IMMR_PITR)

			// Clocks and resets
#define CYGARC_REG_IMM_SCCR                        (CYGARC_REG_IMM_BASE + IMMR_SCCR)
#define CYGARC_REG_IMM_PLPRCR                      (CYGARC_REG_IMM_BASE + IMMR_PLPRCR)
#define CYGARC_REG_IMM_RSR                         (CYGARC_REG_IMM_BASE + IMMR_RSR)
#define CYGARC_REG_IMM_COLIR                       (CYGARC_REG_IMM_BASE + IMMR_COLIR)
#define CYGARC_REG_IMM_VSRMCR                      (CYGARC_REG_IMM_BASE + IMMR_VSRMCR)

			// System IntegrationTimer Keys
#define CYGARC_REG_IMM_TBSCRK                      (CYGARC_REG_IMM_BASE + IMMR_TBSCRK)
#define CYGARC_REG_IMM_TBREF0K                     (CYGARC_REG_IMM_BASE + IMMR_TBREF0K)
#define CYGARC_REG_IMM_TBREF1K                     (CYGARC_REG_IMM_BASE + IMMR_TBREF1K)
#define CYGARC_REG_IMM_TBK                         (CYGARC_REG_IMM_BASE + IMMR_TBK)
#define CYGARC_REG_IMM_RTCSCK                      (CYGARC_REG_IMM_BASE + IMMR_RTCSCK)
#define CYGARC_REG_IMM_RTCK                        (CYGARC_REG_IMM_BASE + IMMR_RTCK)
#define CYGARC_REG_IMM_RTSECK                      (CYGARC_REG_IMM_BASE + IMMR_RTSECK)
#define CYGARC_REG_IMM_RTCALK                      (CYGARC_REG_IMM_BASE + IMMR_RTCALK)
#define CYGARC_REG_IMM_PISCRK                      (CYGARC_REG_IMM_BASE + IMMR_PISCRK)
#define CYGARC_REG_IMM_PITCK                       (CYGARC_REG_IMM_BASE + IMMR_PITCK)

			// Clocks and reset keys
#define CYGARC_REG_IMM_SCCRK                       (CYGARC_REG_IMM_BASE + IMMR_SCCRK)
#define CYGARC_REG_IMM_PLPRCRK                     (CYGARC_REG_IMM_BASE + IMMR_PLPRCRK)
#define CYGARC_REG_IMM_RSRK                        (CYGARC_REG_IMM_BASE + IMMR_RSRK)

		//-------------------------------------
		// CMF (CDR Mone T FLASH EEPROM)
		//-------------------------------------
			// CMF_A
#define CYGARC_REG_IMM_CMFMCR_A                    (CYGARC_REG_IMM_BASE + IMMR_CMFMCR_A)
#define CYGARC_REG_IMM_CMFTST_A                    (CYGARC_REG_IMM_BASE + IMMR_CMFTST_A)
#define CYGARC_REG_IMM_CMFCTL_A                    (CYGARC_REG_IMM_BASE + IMMR_CMFCTL_A)
	
			// CMF_B
#define CYGARC_REG_IMM_CMFMCR_B                    (CYGARC_REG_IMM_BASE + IMMR_CMFMCR_B)
#define CYGARC_REG_IMM_CMFTST_B                    (CYGARC_REG_IMM_BASE + IMMR_CMFTST_B)
#define CYGARC_REG_IMM_CMFCTL_B                    (CYGARC_REG_IMM_BASE + IMMR_CMFCTL_B)

		//-------------------------------------
		// DPTRAM (Dual-Port TPU RAM)
		//-------------------------------------
#define CYGARC_REG_IMM_DPTMCR                      (CYGARC_REG_IMM_BASE + IMMR_DPTMCR)
#define CYGARC_REG_IMM_RAMTST                      (CYGARC_REG_IMM_BASE + IMMR_RAMTST)
#define CYGARC_REG_IMM_RAMBAR                      (CYGARC_REG_IMM_BASE + IMMR_RAMBAR)
#define CYGARC_REG_IMM_MISRH                       (CYGARC_REG_IMM_BASE + IMMR_MISRH)
#define CYGARC_REG_IMM_MISRL                       (CYGARC_REG_IMM_BASE + IMMR_MISRL)
#define CYGARC_REG_IMM_MISCNT                      (CYGARC_REG_IMM_BASE + IMMR_MISCNT)

		//-------------------------------------
		// TPU3 (Time processing unit)
		//-------------------------------------
			// TPU-A
#define CYGARC_REG_IMM_TPUMCR_A                    (CYGARC_REG_IMM_BASE + IMMR_TPUMCR_A)
#define CYGARC_REG_IMM_TCR_A                       (CYGARC_REG_IMM_BASE + IMMR_TCR_A)
#define CYGARC_REG_IMM_DSCR_A                      (CYGARC_REG_IMM_BASE + IMMR_DSCR_A)
#define CYGARC_REG_IMM_DSSR_A                      (CYGARC_REG_IMM_BASE + IMMR_DSSR_A)
#define CYGARC_REG_IMM_TICR_A                      (CYGARC_REG_IMM_BASE + IMMR_TICR_A)
#define CYGARC_REG_IMM_CIER_A                      (CYGARC_REG_IMM_BASE + IMMR_CIER_A)
#define CYGARC_REG_IMM_CFSR0_A                     (CYGARC_REG_IMM_BASE + IMMR_CFSR0_A)
#define CYGARC_REG_IMM_CFSR1_A                     (CYGARC_REG_IMM_BASE + IMMR_CFSR1_A)
#define CYGARC_REG_IMM_CFSR2_A                     (CYGARC_REG_IMM_BASE + IMMR_CFSR2_A)
#define CYGARC_REG_IMM_CFSR3_A                     (CYGARC_REG_IMM_BASE + IMMR_CFSR3_A)
#define CYGARC_REG_IMM_HSQR0_A                     (CYGARC_REG_IMM_BASE + IMMR_HSQR0_A)
#define CYGARC_REG_IMM_HSQR1_A                     (CYGARC_REG_IMM_BASE + IMMR_HSQR1_A)
#define CYGARC_REG_IMM_HSRR0_A                     (CYGARC_REG_IMM_BASE + IMMR_HSRR0_A)
#define CYGARC_REG_IMM_HSRR1_A                     (CYGARC_REG_IMM_BASE + IMMR_HSRR1_A)
#define CYGARC_REG_IMM_CPR0_A                      (CYGARC_REG_IMM_BASE + IMMR_CPR0_A)
#define CYGARC_REG_IMM_CPR1_A                      (CYGARC_REG_IMM_BASE + IMMR_CPR1_A)
#define CYGARC_REG_IMM_CISR_A                      (CYGARC_REG_IMM_BASE + IMMR_CISR_A)
#define CYGARC_REG_IMM_LR_A                        (CYGARC_REG_IMM_BASE + IMMR_LR_A)
#define CYGARC_REG_IMM_SGLR_A                      (CYGARC_REG_IMM_BASE + IMMR_SGLR_A)
#define CYGARC_REG_IMM_DCNR_A                      (CYGARC_REG_IMM_BASE + IMMR_DCNR_A)
#define CYGARC_REG_IMM_TPUMCR2_A                   (CYGARC_REG_IMM_BASE + IMMR_TPUMCR2_A)
#define CYGARC_REG_IMM_TPUMCR3_A                   (CYGARC_REG_IMM_BASE + IMMR_TPUMCR3_A)
#define CYGARC_REG_IMM_ISDR_A                      (CYGARC_REG_IMM_BASE + IMMR_ISDR_A)
#define CYGARC_REG_IMM_ISCR_A                      (CYGARC_REG_IMM_BASE + IMMR_ISCR_A)

              		// TPU-B
#define CYGARC_REG_IMM_TPUMCR_B                    (CYGARC_REG_IMM_BASE + IMMR_TPUMCR_B)
#define CYGARC_REG_IMM_TCR_B                       (CYGARC_REG_IMM_BASE + IMMR_TCR_B)
#define CYGARC_REG_IMM_DSCR_B                      (CYGARC_REG_IMM_BASE + IMMR_DSCR_B)
#define CYGARC_REG_IMM_DSSR_B                      (CYGARC_REG_IMM_BASE + IMMR_DSSR_B)
#define CYGARC_REG_IMM_TICR_B                      (CYGARC_REG_IMM_BASE + IMMR_TICR_B)
#define CYGARC_REG_IMM_CIER_B                      (CYGARC_REG_IMM_BASE + IMMR_CIER_B)
#define CYGARC_REG_IMM_CFSR0_B                     (CYGARC_REG_IMM_BASE + IMMR_CFSR0_B)
#define CYGARC_REG_IMM_CFSR1_B                     (CYGARC_REG_IMM_BASE + IMMR_CFSR1_B)
#define CYGARC_REG_IMM_CFSR2_B                     (CYGARC_REG_IMM_BASE + IMMR_CFSR2_B)
#define CYGARC_REG_IMM_CFSR3_B                     (CYGARC_REG_IMM_BASE + IMMR_CFSR3_B)
#define CYGARC_REG_IMM_HSQR0_B                     (CYGARC_REG_IMM_BASE + IMMR_HSQR0_B)
#define CYGARC_REG_IMM_HSQR1_B                     (CYGARC_REG_IMM_BASE + IMMR_HSQR1_B)
#define CYGARC_REG_IMM_HSRR0_B                     (CYGARC_REG_IMM_BASE + IMMR_HSRR0_B)
#define CYGARC_REG_IMM_HSRR1_B                     (CYGARC_REG_IMM_BASE + IMMR_HSRR1_B)
#define CYGARC_REG_IMM_CPR0_B                      (CYGARC_REG_IMM_BASE + IMMR_CPR0_B)
#define CYGARC_REG_IMM_CPR1_B                      (CYGARC_REG_IMM_BASE + IMMR_CPR1_B)
#define CYGARC_REG_IMM_CISR_B                      (CYGARC_REG_IMM_BASE + IMMR_CISR_B)
#define CYGARC_REG_IMM_LR_B                        (CYGARC_REG_IMM_BASE + IMMR_LR_B)
#define CYGARC_REG_IMM_SGLR_B                      (CYGARC_REG_IMM_BASE + IMMR_SGLR_B)
#define CYGARC_REG_IMM_DCNR_B                      (CYGARC_REG_IMM_BASE + IMMR_DCNR_B)
#define CYGARC_REG_IMM_TPUMCR2_B                   (CYGARC_REG_IMM_BASE + IMMR_TPUMCR2_B)
#define CYGARC_REG_IMM_TPUMCR3_B                   (CYGARC_REG_IMM_BASE + IMMR_TPUMCR3_B)
#define CYGARC_REG_IMM_ISDR_B                      (CYGARC_REG_IMM_BASE + IMMR_ISDR_B)
#define CYGARC_REG_IMM_ISCR_B                      (CYGARC_REG_IMM_BASE + IMMR_ISCR_B)


		//-------------------------------------
		// QADC64 (Queued Analog-to-digital Converter)
		//-------------------------------------
			// QUADC-A
#define CYGARC_REG_IMM_QUADC64MCR_A                (CYGARC_REG_IMM_BASE + IMMR_QUADC64MCR_A)
#define CYGARC_REG_IMM_QUADC64TEST_A               (CYGARC_REG_IMM_BASE + IMMR_QUADC64TEST_A)
#define CYGARC_REG_IMM_QUADC64INT_A                (CYGARC_REG_IMM_BASE + IMMR_QUADC64INT_A)
#define CYGARC_REG_IMM_PORTQA_A_PORTQB_A           (CYGARC_REG_IMM_BASE + IMMR_PORTQA_A_PORTQB_A)
#define CYGARC_REG_IMM_DDRQA_A_DDRQB_A             (CYGARC_REG_IMM_BASE + IMMR_DDRQA_A_DDRQB_A)
#define CYGARC_REG_IMM_QUACR0_A                    (CYGARC_REG_IMM_BASE + IMMR_QUACR0_A)
#define CYGARC_REG_IMM_QUACR1_A                    (CYGARC_REG_IMM_BASE + IMMR_QUACR1_A)
#define CYGARC_REG_IMM_QUACR2_A                    (CYGARC_REG_IMM_BASE + IMMR_QUACR2_A)
#define CYGARC_REG_IMM_QUASR0_A                    (CYGARC_REG_IMM_BASE + IMMR_QUASR0_A)
#define CYGARC_REG_IMM_QUASR1_A                    (CYGARC_REG_IMM_BASE + IMMR_QUASR1_A)
#define CYGARC_REG_IMM_CCW_A                       (CYGARC_REG_IMM_BASE + IMMR_CCW_A)
#define CYGARC_REG_IMM_RJURR_A                     (CYGARC_REG_IMM_BASE + IMMR_RJURR_A)
#define CYGARC_REG_IMM_LJSRR_A                     (CYGARC_REG_IMM_BASE + IMMR_LJSRR_A)
#define CYGARC_REG_IMM_LJURR_A                     (CYGARC_REG_IMM_BASE + IMMR_LJURR_A)

			// QUADC-B
#define CYGARC_REG_IMM_QUADC64MCR_B                (CYGARC_REG_IMM_BASE + IMMR_QUADC64MCR_B)
#define CYGARC_REG_IMM_QUADC64TEST_B               (CYGARC_REG_IMM_BASE + IMMR_QUADC64TEST_B)
#define CYGARC_REG_IMM_QUADC64INT_B                (CYGARC_REG_IMM_BASE + IMMR_QUADC64INT_B)
#define CYGARC_REG_IMM_PORTQA_B_PORTQB_B           (CYGARC_REG_IMM_BASE + IMMR_PORTQA_B_PORTQB_B)
#define CYGARC_REG_IMM_DDRQA_B_DDRQB_B             (CYGARC_REG_IMM_BASE + IMMR_DDRQA_B_DDRQB_B)
#define CYGARC_REG_IMM_QUACR0_B                    (CYGARC_REG_IMM_BASE + IMMR_QUACR0_B)
#define CYGARC_REG_IMM_QUACR1_B                    (CYGARC_REG_IMM_BASE + IMMR_QUACR1_B)
#define CYGARC_REG_IMM_QUACR2_B                    (CYGARC_REG_IMM_BASE + IMMR_QUACR2_B)
#define CYGARC_REG_IMM_QUASR0_B                    (CYGARC_REG_IMM_BASE + IMMR_QUASR0_B)
#define CYGARC_REG_IMM_QUASR1_B                    (CYGARC_REG_IMM_BASE + IMMR_QUASR1_B)
#define CYGARC_REG_IMM_CCW_B                       (CYGARC_REG_IMM_BASE + IMMR_CCW_B)
#define CYGARC_REG_IMM_RJURR_B                     (CYGARC_REG_IMM_BASE + IMMR_RJURR_B)
#define CYGARC_REG_IMM_LJSRR_B                     (CYGARC_REG_IMM_BASE + IMMR_LJSRR_B)
#define CYGARC_REG_IMM_LJURR_B                     (CYGARC_REG_IMM_BASE + IMMR_LJURR_B)

		//-------------------------------------
		// QSMCM (Queued Serial Multi-Channel Module)
		//-------------------------------------
#define CYGARC_REG_IMM_QSMCMMCR                    (CYGARC_REG_IMM_BASE + IMMR_QSMCMMCR)
#define CYGARC_REG_IMM_QTEST                       (CYGARC_REG_IMM_BASE + IMMR_QTEST)
#define CYGARC_REG_IMM_QDSCI_IL                    (CYGARC_REG_IMM_BASE + IMMR_QDSCI_IL)
#define CYGARC_REG_IMM_QSPI_IL                     (CYGARC_REG_IMM_BASE + IMMR_QSPI_IL)
#define CYGARC_REG_IMM_SCC1R0                      (CYGARC_REG_IMM_BASE + IMMR_SCC1R0)
#define CYGARC_REG_IMM_SCC1R1                      (CYGARC_REG_IMM_BASE + IMMR_SCC1R1)
#define CYGARC_REG_IMM_SC1SR                       (CYGARC_REG_IMM_BASE + IMMR_SC1SR)
#define CYGARC_REG_IMM_SC1DR                       (CYGARC_REG_IMM_BASE + IMMR_SC1DR)
#define CYGARC_REG_IMM_PORTQS                      (CYGARC_REG_IMM_BASE + IMMR_PORTQS)
#define CYGARC_REG_IMM_DDRQST                      (CYGARC_REG_IMM_BASE + IMMR_PQSPAR_DDRQST)
#define CYGARC_REG_IMM_SPCR0                       (CYGARC_REG_IMM_BASE + IMMR_SPCR0)
#define CYGARC_REG_IMM_SPCR1                       (CYGARC_REG_IMM_BASE + IMMR_SPCR1)
#define CYGARC_REG_IMM_SPCR2                       (CYGARC_REG_IMM_BASE + IMMR_SPCR2)
#define CYGARC_REG_IMM_SPCR3                       (CYGARC_REG_IMM_BASE + IMMR_SPCR3)
#define CYGARC_REG_IMM_SPSR                        (CYGARC_REG_IMM_BASE + IMMR_SPSR)
#define CYGARC_REG_IMM_SCC2R0                      (CYGARC_REG_IMM_BASE + IMMR_SCC2R0)
#define CYGARC_REG_IMM_SCC2R1                      (CYGARC_REG_IMM_BASE + IMMR_SCC2R1)
#define CYGARC_REG_IMM_SC2SR                       (CYGARC_REG_IMM_BASE + IMMR_SC2SR)
#define CYGARC_REG_IMM_SC2DR                       (CYGARC_REG_IMM_BASE + IMMR_SC2DR)
#define CYGARC_REG_IMM_QSCI1CR                     (CYGARC_REG_IMM_BASE + IMMR_QSCI1CR)
#define CYGARC_REG_IMM_QSCI1SR                     (CYGARC_REG_IMM_BASE + IMMR_QSCI1SR)
#define CYGARC_REG_IMM_SCTQ                        (CYGARC_REG_IMM_BASE + IMMR_SCTQ)
#define CYGARC_REG_IMM_SCRQ                        (CYGARC_REG_IMM_BASE + IMMR_SCRQ)
#define CYGARC_REG_IMM_RECRAM                      (CYGARC_REG_IMM_BASE + IMMR_RECRAM)
#define CYGARC_REG_IMM_TRAN_RAM                    (CYGARC_REG_IMM_BASE + IMMR_TRAN_RAM)
#define CYGARC_REG_IMM_COMD_RAM                    (CYGARC_REG_IMM_BASE + IMMR_COMD_RAM)

		//-------------------------------------
		// MIOS1 (Modular Input/Output Subsystem)
		//-------------------------------------
			// MPIOS Pulse width modulation submodule 0
#define CYGARC_REG_IMM_MPWMSMPERR_0                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_0)
#define CYGARC_REG_IMM_MPWMSMPULR_0                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_0)
#define CYGARC_REG_IMM_MPWMSMCNTR_0                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_0)
#define CYGARC_REG_IMM_MPWMSMSCR_0                 (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_0)
		
			// MPIOS Pulse width modulation submodule 1
#define CYGARC_REG_IMM_MPWMSMPERR_1                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_1)
#define CYGARC_REG_IMM_MPWMSMPULR_1                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_1)
#define CYGARC_REG_IMM_MPWMSMCNTR_1                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_1)
#define CYGARC_REG_IMM_MPWMSMSCR_1                 (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_1)

			// MPIOS Pulse width modulation submodule 2
#define CYGARC_REG_IMM_MPWMSMPERR_2                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_2)
#define CYGARC_REG_IMM_MPWMSMPULR_2                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_2)
#define CYGARC_REG_IMM_MPWMSMCNTR_2                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_2)
#define CYGARC_REG_IMM_MPWMSMSCR_2                 (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_2)

			// MPIOS Pulse width modulation submodule 3
#define CYGARC_REG_IMM_MPWMSMPERR_3                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_3)
#define CYGARC_REG_IMM_MPWMSMPULR_3                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_3)
#define CYGARC_REG_IMM_MPWMSMCNTR_3                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_3)
#define CYGARC_REG_IMM_MPWMSMSCR_3                 (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_3)

			// MIOS Modulus counter submodule 6
#define CYGARC_REG_IMM_MMCSMCNT_6                  (CYGARC_REG_IMM_BASE + IMMR_MMCSMCNT_6)
#define CYGARC_REG_IMM_MMCSMML_6                   (CYGARC_REG_IMM_BASE + IMMR_MMCSMML_6)
#define CYGARC_REG_IMM_MMCSMSCRD_6                 (CYGARC_REG_IMM_BASE + IMMR_MMCSMSCRD_6)
#define CYGARC_REG_IMM_MMCSMSCR_6                  (CYGARC_REG_IMM_BASE + IMMR_MMCSMSCR_6)

			// MIOS Double Action submodule 11
#define CYGARC_REG_IMM_MDASMAR_11                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_11)
#define CYGARC_REG_IMM_MDASMBR_11                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_11)
#define CYGARC_REG_IMM_MDASMSCRD_11                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_11)
#define CYGARC_REG_IMM_MDASMSCR_11                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_11)

			// MIOS Double Action submodule 12
#define CYGARC_REG_IMM_MDASMAR_12                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_12)
#define CYGARC_REG_IMM_MDASMBR_12                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_12)
#define CYGARC_REG_IMM_MDASMSCRD_12                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_12)
#define CYGARC_REG_IMM_MDASMSCR_12                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_12)

			// MIOS Double Action submodule 13
#define CYGARC_REG_IMM_MDASMAR_13                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_13)
#define CYGARC_REG_IMM_MDASMBR_13                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_13)
#define CYGARC_REG_IMM_MDASMSCRD_13                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_13)
#define CYGARC_REG_IMM_MDASMSCR_13                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_13)

			// MIOS Double Action submodule 14
#define CYGARC_REG_IMM_MDASMAR_14                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_14)
#define CYGARC_REG_IMM_MDASMBR_14                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_14)
#define CYGARC_REG_IMM_MDASMSCRD_14                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_14)
#define CYGARC_REG_IMM_MDASMSCR_14                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_14)

			// MIOS Double Action submodule 15
#define CYGARC_REG_IMM_MDASMAR_15                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_15)
#define CYGARC_REG_IMM_MDASMBR_15                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_15)
#define CYGARC_REG_IMM_MDASMSCRD_15                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_15)
#define CYGARC_REG_IMM_MDASMSCR_15                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_15)

			// MPIOS Pulse width modulation submodule 16
#define CYGARC_REG_IMM_MPWMSMPERR_16               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_16)
#define CYGARC_REG_IMM_MPWMSMPULR_16               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_16)
#define CYGARC_REG_IMM_MPWMSMCNTR_16               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_16)
#define CYGARC_REG_IMM_MPWMSMSCR_16                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_16)

			// MPIOS Pulse width modulation submodule 17
#define CYGARC_REG_IMM_MPWMSMPERR_17               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_17)
#define CYGARC_REG_IMM_MPWMSMPULR_17               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_17)
#define CYGARC_REG_IMM_MPWMSMCNTR_17               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_17)
#define CYGARC_REG_IMM_MPWMSMSCR_17                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_17)

			// MPIOS Pulse width modulation submodule 18
#define CYGARC_REG_IMM_MPWMSMPERR_18               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_18)
#define CYGARC_REG_IMM_MPWMSMPULR_18               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_18)
#define CYGARC_REG_IMM_MPWMSMCNTR_18               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_18)
#define CYGARC_REG_IMM_MPWMSMSCR_18                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_18)

			// MPIOS Pulse width modulation submodule 19
#define CYGARC_REG_IMM_MPWMSMPERR_19               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPERR_19)
#define CYGARC_REG_IMM_MPWMSMPULR_19               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMPULR_19)
#define CYGARC_REG_IMM_MPWMSMCNTR_19               (CYGARC_REG_IMM_BASE + IMMR_MPWMSMCNTR_19)
#define CYGARC_REG_IMM_MPWMSMSCR_19                (CYGARC_REG_IMM_BASE + IMMR_MPWMSMSCR_19)

			// MIOS Modulus counter submodule 22
#define CYGARC_REG_IMM_MMCSMCNT_22                 (CYGARC_REG_IMM_BASE + IMMR_MMCSMCNT_22)
#define CYGARC_REG_IMM_MMCSMML_22                  (CYGARC_REG_IMM_BASE + IMMR_MMCSMML_22)
#define CYGARC_REG_IMM_MMCSMSCRD_22                (CYGARC_REG_IMM_BASE + IMMR_MMCSMSCRD_22)
#define CYGARC_REG_IMM_MMCSMSCR_22                 (CYGARC_REG_IMM_BASE + IMMR_MMCSMSCR_22)

			// MIOS Double action submodule 27
#define CYGARC_REG_IMM_MDASMAR_27                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_27)
#define CYGARC_REG_IMM_MDASMBR_27                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_27)
#define CYGARC_REG_IMM_MDASMSCRD_27                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_27)
#define CYGARC_REG_IMM_MDASMSCR_27                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_27)

			// MIOS Double action submodule 28
#define CYGARC_REG_IMM_MDASMAR_28                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_28)
#define CYGARC_REG_IMM_MDASMBR_28                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_28)
#define CYGARC_REG_IMM_MDASMSCRD_28                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_28)
#define CYGARC_REG_IMM_MDASMSCR_28                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_28)

			// MIOS Double action submodule 29
#define CYGARC_REG_IMM_MDASMAR_29                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_29)
#define CYGARC_REG_IMM_MDASMBR_29                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_29)
#define CYGARC_REG_IMM_MDASMSCRD_29                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_29)
#define CYGARC_REG_IMM_MDASMSCR_29                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_29)

			// MIOS Double action submodule 30
#define CYGARC_REG_IMM_MDASMAR_30                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_30)
#define CYGARC_REG_IMM_MDASMBR_30                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_30)
#define CYGARC_REG_IMM_MDASMSCRD_30                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_30)
#define CYGARC_REG_IMM_MDASMSCR_30                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_30)

			// MIOS Double action submodule 31
#define CYGARC_REG_IMM_MDASMAR_31                  (CYGARC_REG_IMM_BASE + IMMR_MDASMAR_31)
#define CYGARC_REG_IMM_MDASMBR_31                  (CYGARC_REG_IMM_BASE + IMMR_MDASMBR_31)
#define CYGARC_REG_IMM_MDASMSCRD_31                (CYGARC_REG_IMM_BASE + IMMR_MDASMSCRD_31)
#define CYGARC_REG_IMM_MDASMSCR_31                 (CYGARC_REG_IMM_BASE + IMMR_MDASMSCR_31)

			// MIOS Paralell port I/O submodule
#define CYGARC_REG_IMM_MPIOSMDR                    (CYGARC_REG_IMM_BASE + IMMR_MPIOSMDR)
#define CYGARC_REG_IMM_MPIOSMDDR                   (CYGARC_REG_IMM_BASE + IMMR_MPIOSMDDR)

			// MIOS Bus interface Submodule
#define CYGARC_REG_IMM_MPIOS1TPCR                  (CYGARC_REG_IMM_BASE + IMMR_MIOS1TPCR)
#define CYGARC_REG_IMM_MPIOS1VNR                   (CYGARC_REG_IMM_BASE + IMMR_MIOS1VNR)
#define CYGARC_REG_IMM_MIOS1MCR                    (CYGARC_REG_IMM_BASE + IMMR_MIOS1MCR)

			// MIOS Counter / Prescaler submodule
#define CYGARC_REG_IMM_MCPSMSCR                    (CYGARC_REG_IMM_BASE + IMMR_MCPSMSCR)

			// MIOS Interrupt request submodule 0
#define CYGARC_REG_IMM_MIOS1SR0                    (CYGARC_REG_IMM_BASE + IMMR_MIOS1SR0)
#define CYGARC_REG_IMM_MIOS1ER0                    (CYGARC_REG_IMM_BASE + IMMR_MIOS1ER0)
#define CYGARC_REG_IMM_MIOS1RPR0                   (CYGARC_REG_IMM_BASE + IMMR_MIOS1RPR0)
#define CYGARC_REG_IMM_MIOS1LVL0                   (CYGARC_REG_IMM_BASE + IMMR_MIOS1LVL0)

			// Mios Interrupt request submodule 1
#define CYGARC_REG_IMM_MIOS1SR1                    (CYGARC_REG_IMM_BASE + IMMR_MIOS1SR1)
#define CYGARC_REG_IMM_MIOS1ER1                    (CYGARC_REG_IMM_BASE + IMMR_MIOS1ER1)
#define CYGARC_REG_IMM_MIOS1RPR1                   (CYGARC_REG_IMM_BASE + IMMR_MIOS1RPR1)
#define CYGARC_REG_IMM_MIOS1LVL1                   (CYGARC_REG_IMM_BASE + IMMR_MIOS1LVL1)

		//-------------------------------------
		// TouCAN (CAN 2.0B Controller)
		//-------------------------------------
#define CYGARC_REG_IMM_CANICR_IRL                  0x0700
#define CYGARC_REG_IMM_CANICR_IRL_SHIFT            8
#define CYGARC_REG_IMM_CANICR_ILBS                 0x00c0
#define CYGARC_REG_IMM_CANICR_ILBS_SHIFT           6

#define CYGARC_REG_IMM_TCNMCR_STOP                 0x8000
#define CYGARC_REG_IMM_TCNMCR_FRZ                  0x4000
#define CYGARC_REG_IMM_TCNMCR_HALT                 0x1000
#define CYGARC_REG_IMM_TCNMCR_NOTRDY               0x0800
#define CYGARC_REG_IMM_TCNMCR_WAKEMSK              0x0400
#define CYGARC_REG_IMM_TCNMCR_SOFTRST              0x0200
#define CYGARC_REG_IMM_TCNMCR_FRZACK               0x0100
#define CYGARC_REG_IMM_TCNMCR_SUPV                 0x0080
#define CYGRAC_REG_IMM_TCNMCR_SELFWAKE             0x0040
#define CYGARC_REG_IMM_TCNMCR_APS                  0x0020
#define CYGARC_REG_IMM_TCNMCR_STOPACK              0x0010

#define CYGARC_REG_IMM_CANCTRL0_CANCTRL1_BOFFMSK   0x8000
#define CYGARC_REG_IMM_CANCTRL0_CANCTRL1_ERRMSK    0x4000
#define CYGARC_REG_IMM_CANCTRL0_CANCTRL1_RXMOD     0x0c00
#define CYGARC_REG_IMM_CANCTRL0_CANCTRL1_TXMOD     0x0300
#define CYGARC_REG_IMM_CANCTRL0_CANCTRL1_RXMOD_SHIFT   10
#define CYGARC_REG_IMM_CANCTRL0_CANCTRL1_TXMOD_SHIFT    8

#define CYGARC_REG_IMM_ESTAT_BITERR                0xc000
#define CYGARC_REG_IMM_ESTAT_ACKERR                0x2000
#define CYGARC_REG_IMM_ESTAT_CRCERR                0x1000
#define CYGARC_REG_IMM_ESTAT_FORMERR               0x0800
#define CYGARC_REG_IMM_ESTAT_STUFFERR              0x0400
#define CYGARC_REG_IMM_ESTAT_TXWARN                0x0200
#define CYGARC_REG_IMM_ESTAT_RXWARN                0x0100
#define CYGARC_REG_IMM_ESTAT_IDLE                  0x0080
#define CYGARC_REG_IMM_ESTAT_TX_RX                 0x0040
#define CYGARC_REG_IMM_ESTAT_FCS                   0x0030
#define CYGARC_REG_IMM_ESTAT_BOFFINT               0x0004
#define CYGARC_REG_IMM_ESTAT_ERRINT                0x0002
#define CYGARC_REG_IMM_ESTAT_WAKEINT               0x0001
#define CYGARC_REG_IMM_ESTAT_BITERR_SHIFT              14
#define CYGARC_REG_IMM_ESTAT_FCS_SHIFT                  4

			// TouCAN_A
#define CYGARC_REG_IMM_TCNMCR_A                    (CYGARC_REG_IMM_BASE + IMMR_TCNMCR_A)
#define CYGARC_REG_IMM_TTR_A                       (CYGARC_REG_IMM_BASE + IMMR_TTR_A)
#define CYGARC_REG_IMM_CANICR_A                    (CYGARC_REG_IMM_BASE + IMMR_CANICR_A)
#define CYGARC_REG_IMM_CANCTRL0_A_CANCTRL1_A       (CYGARC_REG_IMM_BASE + IMMR_CANCTRL0_A_CANCTRL1_A)
#define CYGARC_REG_IMM_PRESDIV_A_CTRL2_A           (CYGARC_REG_IMM_BASE + IMMR_PRESDIV_A_CTRL2_A)
#define CYGARC_REG_IMM_TIMER_A                     (CYGARC_REG_IMM_BASE + IMMR_TIMER_A)
#define CYGARC_REG_IMM_RXGMASKHI_A                 (CYGARC_REG_IMM_BASE + IMMR_RXGMASKHI_A)
#define CYGARC_REG_IMM_RXGMASKLO_A                 (CYGARC_REG_IMM_BASE + IMMR_RXGMASKLO_A)
#define CYGARC_REG_IMM_RX14MASKHI_A                (CYGARC_REG_IMM_BASE + IMMR_RX14MASKHI_A)
#define CYGARC_REG_IMM_RX14MASKLO_A                (CYGARC_REG_IMM_BASE + IMMR_RX14MASKLO_A)
#define CYGARC_REG_IMM_RX15MASKHI_A                (CYGARC_REG_IMM_BASE + IMMR_RX15MASKHI_A)
#define CYGARC_REG_IMM_RX15MASKLO_A                (CYGARC_REG_IMM_BASE + IMMR_RX15MASKLO_A)
#define CYGARC_REG_IMM_ESTAT_A                     (CYGARC_REG_IMM_BASE + IMMR_ESTAT_A)
#define CYGARC_REG_IMM_IMASK_A                     (CYGARC_REG_IMM_BASE + IMMR_IMASK_A)
#define CYGARC_REG_IMM_IFLAG_A                     (CYGARC_REG_IMM_BASE + IMMR_IFLAG_A)
#define CYGARC_REG_IMM_RXECTR_A_TXECTR_A           (CYGARC_REG_IMM_BASE + IMMR_RXECTR_A_TXECTR_A)

			// TouCAN_B
#define CYGARC_REG_IMM_TCNMCR_B                    (CYGARC_REG_IMM_BASE + IMMR_TCNMCR_B)
#define CYGARC_REG_IMM_TTR_B                       (CYGARC_REG_IMM_BASE + IMMR_TTR_B)
#define CYGARC_REG_IMM_CANICR_B                    (CYGARC_REG_IMM_BASE + IMMR_CANICR_B)
#define CYGARC_REG_IMM_CANCTRL0_B_CANCTRL1_B       (CYGARC_REG_IMM_BASE + IMMR_CANCTRL0_B_CANCTRL1_B)
#define CYGARC_REG_IMM_PRESDIV_B_CTRL2_B           (CYGARC_REG_IMM_BASE + IMMR_PRESDIV_B_CTRL2_B)
#define CYGARC_REG_IMM_TIMER_B                     (CYGARC_REG_IMM_BASE + IMMR_TIMER_B)
#define CYGARC_REG_IMM_RXGMASKHI_B                 (CYGARC_REG_IMM_BASE + IMMR_RXGMASKHI_B)
#define CYGARC_REG_IMM_RXGMASKLO_B                 (CYGARC_REG_IMM_BASE + IMMR_RXGMASKLO_B)
#define CYGARC_REG_IMM_RX14MASKHI_B                (CYGARC_REG_IMM_BASE + IMMR_RX14MASKHI_B)
#define CYGARC_REG_IMM_RX14MASKLO_B                (CYGARC_REG_IMM_BASE + IMMR_RX14MASKLO_B)
#define CYGARC_REG_IMM_RX15MASKHI_B                (CYGARC_REG_IMM_BASE + IMMR_RX15MASKHI_B)
#define CYGARC_REG_IMM_RX15MASKLO_B                (CYGARC_REG_IMM_BASE + IMMR_RX15MASKLO_B)
#define CYGARC_REG_IMM_ESTAT_B                     (CYGARC_REG_IMM_BASE + IMMR_ESTAT_B)
#define CYGARC_REG_IMM_IMASK_B                     (CYGARC_REG_IMM_BASE + IMMR_IMASK_B)
#define CYGARC_REG_IMM_IFLAG_B                     (CYGARC_REG_IMM_BASE + IMMR_IFLAG_B)
#define CYGARC_REG_IMM_RXECTR_B_TXECTR_B           (CYGARC_REG_IMM_BASE + IMMR_RXECTR_A_TXECTR_B)

		//-------------------------------------
		// UIMB (U-Bus to IMB3 Bus Interface)
		//-------------------------------------
#define CYGARC_REG_IMM_UMCR                        (CYGARC_REG_IMM_BASE + IMMR_UMCR)
#define CYGARC_REG_IMM_UTSTCREG                    (CYGARC_REG_IMM_BASE + IMMR_UTSTCREG)
#define CYGARC_REG_IMM_UIPEND                      (CYGARC_REG_IMM_BASE + IMMR_UIPEND)

		//-------------------------------------
		// SRAM (Static RAM Access memory)
		//-------------------------------------
#define CYGARC_REG_IMM_SRAMMCR_A                   (CYGARC_REG_IMM_BASE + IMMR_SRAMMCR_A)
#define CYGARC_REG_IMM_SRAMTST_A                   (CYGARC_REG_IMM_BASE + IMMR_SRAMTST_A)
#define CYGARC_REG_IMM_SRAMMCR_B                   (CYGARC_REG_IMM_BASE + IMMR_SRAMMCR_B)
#define CYGARC_REG_IMM_SRAMTST_B                   (CYGARC_REG_IMM_BASE + IMMR_SRAMTST_B)

//---------------------------------------------------------------------------------
// Real Time Clock access macros
#define CYGARC_RTCSETALARM(_v_) \
  RTCAL = _v_;
  
#define CYGARC_RTCRESET    \
  RTC = 0;

//---------------------------------------------------------------------------------
// Locking and unlocking registers
#define CYGARC_LOCKTIMEBASE \
  TBK = 0;

#define CYGARC_UNLOCKTIMEBASE \
  TBK = 0x55ccaa33;

#define CYGARC_LOCKREGISTER(_reg_) \
  switch(_reg_) {                  \
    case CYGARC_REG_IMM_TBSCR:     \
      TBSCRK = 0;                  \
      break;                       \
    case CYGARC_REG_IMM_TBREF0:    \
      TBREF0K = 0;                 \
      break;                       \
    case CYGARC_REG_IMM_TBREF1:    \
      TBREF1K = 0;                 \
      break;                       \
    case CYGARC_REG_IMM_RTCSC:     \
      RTCSCK = 0;                  \
      break;                       \
    case CYGARC_REG_IMM_RTC:       \
      RTCK = 0;                    \
      break;                       \
    case CYGARC_REG_IMM_RTSEC:     \
      RTSECK = 0;                  \
      break;                       \
    case CYGARC_REG_IMM_RTCAL:     \
      RTCALK = 0;                  \
      break;                       \
    case CYGARC_REG_IMM_PISCR:     \
      PISCRK = 0;                  \
      break;                       \
    case CYGARC_REG_IMM_PITC:      \
      PITCK = 0;                   \
      break;                       \
    case CYGARC_REG_IMM_SCCR:      \
      SCCRK = 0;                   \
      break;                       \
    case CYGARC_REG_IMM_PLPRCR:    \
      PLPRCRK = 0;                 \
      break;                       \
    case CYGARC_REG_IMM_RSR:       \
      RSRK = 0;                    \
      break;                       \
    default:                       \
      break;                       \
  }

#define CYGARC_UNLOCKREGISTER(_reg_) \
  switch(_reg_)                      \
  {                                  \
    case CYGARC_REG_IMM_TBSCR:       \
      TBSCRK = 0x55ccaa33;           \
      break;                         \
    case CYGARC_REG_IMM_TBREF0:      \
      TBREF0K = 0x55ccaa33;          \
      break;                         \
    case CYGARC_REG_IMM_TBREF1:      \
      TBREF1K = 0x55ccaa33;          \
      break;                         \
    case CYGARC_REG_IMM_RTCSC:       \
      RTCSCK = 0x55ccaa33;           \
      break;                         \
    case CYGARC_REG_IMM_RTC:         \
      RTCK = 0x55ccaa33;             \
      break;                         \
    case CYGARC_REG_IMM_RTSEC:       \
      RTSECK = 0x55ccaa33;           \
      break;                         \
    case CYGARC_REG_IMM_RTCAL:       \
      RTCALK = 0x55ccaa33;           \
      break;                         \
    case CYGARC_REG_IMM_PISCR:       \
      PISCRK = 0x55ccaa33;           \
      break;                         \
    case CYGARC_REG_IMM_PITC:        \
      PITCK = 0x55ccaa33;            \
      break;                         \
    case CYGARC_REG_IMM_SCCR:        \
      SCCRK = 0x55ccaa33;            \
      break;                         \
    case CYGARC_REG_IMM_PLPRCR:      \
      PLPRCRK = 0x55ccaa33;          \
      break;                         \
    case CYGARC_REG_IMM_RSR:         \
      RSRK = 0x55ccaa33;             \
      break;                         \
    default:                         \
      break;                         \
  }

//---------------------------------------------------------------------------------
// Periodic interrupt timer access macros
#define CYGARC_SETPITCOUNT(_v_) \
  PITC = ((cyg_uint32)_v_ << CYGARC_REG_IMM_PITC_COUNT_SHIFT);

#endif // CYGONCE_HAL_MPC555_REGS_H
