##=============================================================================
##
##      context.s
##
##      MPC555 context switch code
##
##=============================================================================

#include <pkgconf/hal.h>
#include "cyg/hal/mpc555.inc"

#------------------------------------------------------------------------------
# function declaration macro
                
#define FUNC_START(name) \
        .type name,@function; \
        .globl name; \
name:   

#------------------------------------------------------------------------------
# Configure to use either a minimal or maximal thread state
#ifdef CYGDBG_HAL_COMMON_CONTEXT_SAVE_MINIMUM

#define MIN_SAVE_REG    14
#define MAX_SAVE_REG    31

#define MIN_SAVE_FREG   14
#define MAX_SAVE_FREG   31

        .macro  save_special
        .endm

        .macro  load_special
        .endm
#else

#define MIN_SAVE_REG     6
#define MAX_SAVE_REG    31

#define MIN_SAVE_FREG    0
#define MAX_SAVE_FREG   31

        .macro  save_special
        mfxer   r6
        mfctr   r7
        stw     r6,cont_ppcreg_xer(sp)
        stw     r7,cont_ppcreg_ctr(sp)
        .endm

        .macro  load_special
        lwz     r6,cont_ppcreg_xer(sp)
        lwz     r7,cont_ppcreg_ctr(sp)
        mtxer   r6
        mtctr   r7
        .endm
                
#endif                  
                
#------------------------------------------------------------------------------
# hal_thread_switch_context
# Switch thread contexts
# R3 = address of sp of next thread to execute
# R4 = address of sp save location of current thread

FUNC_START(hal_thread_switch_context)
        mr      r5,sp                       # R5 = saved stack pointer
        subi    sp,sp,ppcreg_context_size   # space for state

        # Save registers MIN..MAX
        .set    _reg, MIN_SAVE_REG
        .rept   MAX_SAVE_REG+1-MIN_SAVE_REG
        stw     _reg,cont_ppcreg_reg+(_reg-MIN_SAVE_REG)*4(sp)
        .set    _reg,_reg+1
        .endr

	# Save floating point registers MIN..MAX
	.set	_freg, MIN_SAVE_FREG 
	.rept	MAX_SAVE_FREG+1-MIN_SAVE_FREG
	stfd	_freg,cont_ppcreg_freg+(_freg-MIN_SAVE_FREG)*8(sp)
	.set	_freg, _freg+1
	.endr

        stw     r0,cont_ppcreg_r0(sp)      # R0
        stw     r5,cont_ppcreg_r1(sp)      # R5 = real SP, save in R1 slot
        stw     r2,cont_ppcreg_r2(sp)      # R2 = TOC

        # CR cannot be treated as a special register since GCC will only
        # do partial restore of CR at function exit, depending on which
        # of CR2, CR3, and CR4 have been used in the given function.
        mfcr    r5                          # save CR.
        stw     r5,cont_ppcreg_cr(sp)

#ifdef CYGDBG_HAL_DEBUG_GDB_THREAD_SUPPORT
        # Make the thread context look like an exception context if thread-
        # aware debugging is required. This state does not need restoring.
        mflr    r5
        stw     r5,cont_ppcreg_pc(sp)            # pc of caller
        mfmsr   r5
        stw     r5,cont_ppcreg_msr(sp)           # msr (or PS)
#endif

        save_special                        # save special regs
        
        mflr    r5                          # save LR == return link for this fn
        stw     r5,cont_ppcreg_lr(sp)
        
        stw     sp,0(r4)                    # save SP into save location

        # Now load the destination thread by dropping through
        # to hal_thread_load_context
        
#------------------------------------------------------------------------------
# hal_thread_load_context
# Load thread context
# R3 = address of sp of next thread to execute
# Note that this function is also the second half of hal_thread_switch_context
# and is simply dropped into from it.
        
FUNC_START(hal_thread_load_context)
        
        lwz     sp,0(r3)                    # load state directly into SP

        load_special                        # load special registers

        lwz     r5,cont_ppcreg_cr(sp)       # restore CR
        mtcr    r5

        lwz     r5,cont_ppcreg_lr(sp)       # get LR as set as return link
        mtlr    r5
                
        # Load registers MIN..MAX
        .set    _reg,MIN_SAVE_REG
        .rept   MAX_SAVE_REG+1-MIN_SAVE_REG
        lwz     _reg,cont_ppcreg_reg+(_reg-MIN_SAVE_REG)*4(sp)
        .set    _reg,_reg+1
        .endr

	# Load floating point registers MIN..MAX
	.set	_freg, MIN_SAVE_FREG
	.rept	MAX_SAVE_FREG+1-MIN_SAVE_FREG
	lfd	_freg,cont_ppcreg_freg+(_freg-MIN_SAVE_FREG)*8(sp)
	.set	_freg,_freg+1
	.endr

        lwz     r0,cont_ppcreg_r0(sp)       # R0
        lwz     r2,cont_ppcreg_r2(sp)       # R2 = TOC
        lwz     r3,cont_ppcreg_r3(sp)       # load r3
        
        lwz     sp,cont_ppcreg_r1(sp)       # finally restore true SP
        
        blr                                 # jump to LR
                        
#------------------------------------------------------------------------------
# HAL longjmp, setjmp implementations
# hal_setjmp saves only to callee save registers 14-31, r1[sp],r2, cr[2-4], f14-f31
# and lr into buffer supplied in r3[arg0]
# Note: These definitions are repeated in hal_arch.h. If changes are required
# remember to update both sets.

#define CYGARC_JMP_BUF_SP        0
#define CYGARC_JMP_BUF_R2        1
#define CYGARC_JMP_BUF_R14       2
#define CYGARC_JMP_BUF_R15       3
#define CYGARC_JMP_BUF_R16       4
#define CYGARC_JMP_BUF_R17       5
#define CYGARC_JMP_BUF_R18       6
#define CYGARC_JMP_BUF_R19       7
#define CYGARC_JMP_BUF_R20       8
#define CYGARC_JMP_BUF_R21       9
#define CYGARC_JMP_BUF_R22      10
#define CYGARC_JMP_BUF_R23      11
#define CYGARC_JMP_BUF_R24      12
#define CYGARC_JMP_BUF_R25      13
#define CYGARC_JMP_BUF_R26      14
#define CYGARC_JMP_BUF_R27      15
#define CYGARC_JMP_BUF_R28      16
#define CYGARC_JMP_BUF_R29      17
#define CYGARC_JMP_BUF_R30      18
#define CYGARC_JMP_BUF_R31      19
#define CYGARC_JMP_BUF_LR       20
#define CYGARC_JMP_BUF_CR       21
#define CYGARC_JMP_BUF_F14	22
#define CYGARC_JMP_BUF_F15	24
#define CYGARC_JMP_BUF_F16	26
#define CYGARC_JMP_BUF_F17	28
#define CYGARC_JMP_BUF_F18	30
#define CYGARC_JMP_BUF_F19	32
#define CYGARC_JMP_BUF_F20	34
#define CYGARC_JMP_BUF_F21	36
#define CYGARC_JMP_BUF_F22	38
#define CYGARC_JMP_BUF_F23	40
#define CYGARC_JMP_BUF_F24	42
#define CYGARC_JMP_BUF_F25	44
#define CYGARC_JMP_BUF_F26	46
#define CYGARC_JMP_BUF_F27	48
#define CYGARC_JMP_BUF_F28	50
#define CYGARC_JMP_BUF_F29	52
#define CYGARC_JMP_BUF_F30	54
#define CYGARC_JMP_BUF_F31	56

#define CYGARC_JMP_BUF_SIZE     58

FUNC_START(hal_setjmp)
        mfcr    r5
        stw     r5, CYGARC_JMP_BUF_CR*4(r3)
        mflr    r5
        stw     r5, CYGARC_JMP_BUF_LR*4(r3)
        stw     r31,CYGARC_JMP_BUF_R31*4(r3)
        stw     r30,CYGARC_JMP_BUF_R30*4(r3)
        stw     r29,CYGARC_JMP_BUF_R29*4(r3)
        stw     r28,CYGARC_JMP_BUF_R28*4(r3)
        stw     r27,CYGARC_JMP_BUF_R27*4(r3)
        stw     r26,CYGARC_JMP_BUF_R26*4(r3)
        stw     r25,CYGARC_JMP_BUF_R25*4(r3)
        stw     r24,CYGARC_JMP_BUF_R24*4(r3)
        stw     r23,CYGARC_JMP_BUF_R23*4(r3)
        stw     r22,CYGARC_JMP_BUF_R22*4(r3)
        stw     r21,CYGARC_JMP_BUF_R21*4(r3)
        stw     r20,CYGARC_JMP_BUF_R20*4(r3)
        stw     r19,CYGARC_JMP_BUF_R19*4(r3)
        stw     r18,CYGARC_JMP_BUF_R18*4(r3)
        stw     r17,CYGARC_JMP_BUF_R17*4(r3)
        stw     r16,CYGARC_JMP_BUF_R16*4(r3)
        stw     r15,CYGARC_JMP_BUF_R15*4(r3)
        stw     r14,CYGARC_JMP_BUF_R14*4(r3)
        stw     r2, CYGARC_JMP_BUF_R2*4(r3)        # TOC, optimize out?
        stw     sp, CYGARC_JMP_BUF_SP*4(r3)
	stfd	f31, CYGARC_JMP_BUF_F31*4(r3)
        stfd	f30, CYGARC_JMP_BUF_F30*4(r3)
        stfd	f29, CYGARC_JMP_BUF_F29*4(r3)
        stfd	f28, CYGARC_JMP_BUF_F28*4(r3)
        stfd	f27, CYGARC_JMP_BUF_F27*4(r3)
        stfd	f26, CYGARC_JMP_BUF_F26*4(r3)
        stfd	f25, CYGARC_JMP_BUF_F25*4(r3)
        stfd	f24, CYGARC_JMP_BUF_F24*4(r3)
        stfd	f23, CYGARC_JMP_BUF_F23*4(r3)
        stfd	f22, CYGARC_JMP_BUF_F22*4(r3)
        stfd	f21, CYGARC_JMP_BUF_F21*4(r3)
        stfd	f20, CYGARC_JMP_BUF_F20*4(r3)
        stfd	f19, CYGARC_JMP_BUF_F19*4(r3)
        stfd	f18, CYGARC_JMP_BUF_F18*4(r3)
        stfd	f17, CYGARC_JMP_BUF_F17*4(r3)
        stfd	f16, CYGARC_JMP_BUF_F16*4(r3)
        stfd	f15, CYGARC_JMP_BUF_F15*4(r3)
        stfd	f14, CYGARC_JMP_BUF_F14*4(r3)
        li      r3,0            # return 0
        blr

# hal_longjmp loads state from r3[arg0] and returns
# and lr into buffer supplied in r3[arg0]

FUNC_START(hal_longjmp)
        lwz     r5,CYGARC_JMP_BUF_CR*4(r3)
        mtcr    r5
        lwz     r5,CYGARC_JMP_BUF_LR*4(r3)
        mtlr    r5
        lwz     r31,CYGARC_JMP_BUF_R31*4(r3)
        lwz     r30,CYGARC_JMP_BUF_R30*4(r3)
        lwz     r29,CYGARC_JMP_BUF_R29*4(r3)
        lwz     r28,CYGARC_JMP_BUF_R28*4(r3)
        lwz     r27,CYGARC_JMP_BUF_R27*4(r3)
        lwz     r26,CYGARC_JMP_BUF_R26*4(r3)
        lwz     r25,CYGARC_JMP_BUF_R25*4(r3)
        lwz     r24,CYGARC_JMP_BUF_R24*4(r3)
        lwz     r23,CYGARC_JMP_BUF_R23*4(r3)
        lwz     r22,CYGARC_JMP_BUF_R22*4(r3)
        lwz     r21,CYGARC_JMP_BUF_R21*4(r3)
        lwz     r20,CYGARC_JMP_BUF_R20*4(r3)
        lwz     r19,CYGARC_JMP_BUF_R19*4(r3)
        lwz     r18,CYGARC_JMP_BUF_R18*4(r3)
        lwz     r17,CYGARC_JMP_BUF_R17*4(r3)
        lwz     r16,CYGARC_JMP_BUF_R16*4(r3)
        lwz     r15,CYGARC_JMP_BUF_R15*4(r3)
        lwz     r14,CYGARC_JMP_BUF_R14*4(r3)
        lwz     r2, CYGARC_JMP_BUF_R2*4(r3)
        lwz     sp, CYGARC_JMP_BUF_SP*4(r3)
        mr      r3,r4           # return r4[arg1]
	lfd	f31, CYGARC_JMP_BUF_F31*4(r3)
	lfd	f30, CYGARC_JMP_BUF_F30*4(r3)
	lfd	f29, CYGARC_JMP_BUF_F29*4(r3)
	lfd	f28, CYGARC_JMP_BUF_F28*4(r3)
	lfd	f27, CYGARC_JMP_BUF_F27*4(r3)
	lfd	f26, CYGARC_JMP_BUF_F26*4(r3)
	lfd	f25, CYGARC_JMP_BUF_F25*4(r3)
	lfd	f24, CYGARC_JMP_BUF_F24*4(r3)
	lfd	f23, CYGARC_JMP_BUF_F23*4(r3)
	lfd	f22, CYGARC_JMP_BUF_F22*4(r3)
	lfd	f21, CYGARC_JMP_BUF_F21*4(r3)
	lfd	f20, CYGARC_JMP_BUF_F20*4(r3)
	lfd	f19, CYGARC_JMP_BUF_F19*4(r3)
	lfd	f18, CYGARC_JMP_BUF_F18*4(r3)
	lfd	f17, CYGARC_JMP_BUF_F17*4(r3)
	lfd	f16, CYGARC_JMP_BUF_F16*4(r3)
	lfd	f15, CYGARC_JMP_BUF_F15*4(r3)
	lfd	f14, CYGARC_JMP_BUF_F14*4(r3)
        blr

#------------------------------------------------------------------------------
# end of context.s
