#ifndef TB_ISRDISPATCH_CPP
#define TB_ISRDISPATCH_CPP

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/hal/tb_isrdispatch.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/mpc555_regs.h"
#include "cyg/hal/hal_io.h"
#include "cyg/hal/hal_intr.h"

//---------------------
// Class implementation
//---------------------
// Public constructor and destructor
TimeBase_isrDispatcher::TimeBase_isrDispatcher()
    : HAL_isrDispatcher()
{
}

// Provide an (empty) virtual destructor
TimeBase_isrDispatcher::~TimeBase_isrDispatcher()
{
}

//------------------------------------------
// Protected member functions implementation
//------------------------------------------
// set InterruptingLevel
bool TimeBase_isrDispatcher::setInterruptingLevel(unsigned char a_level)
{
  cyg_uint16 tbscr;
  cyg_uint16 level = (CYGARC_REG_IMM_TBSCR_IRQ0 >> a_level);

  HAL_READ_UINT16(CYGARC_REG_IMM_TBSCR, tbscr);
  tbscr &= ~((cyg_uint16)CYGARC_REG_IMM_TBSCR_IRQMASK);
  tbscr |= level;
  tbscr &= ~(CYGARC_REG_IMM_TBSCR_REFA); // Prevent from clearing interrupts flags
  tbscr &= ~(CYGARC_REG_IMM_TBSCR_REFB); // accidently. Just do what is asked.
  HAL_WRITE_UINT16(CYGARC_REG_IMM_TBSCR, tbscr);

  return true;
}

// getInterruptingVector
cyg_uint32 TimeBase_isrDispatcher::findInterruptingVector(void)
{
  cyg_uint16 tbscr;
  
  HAL_READ_UINT16(CYGARC_REG_IMM_TBSCR, tbscr);

  if((tbscr & CYGARC_REG_IMM_TBSCR_REFA) && (tbscr & CYGARC_REG_IMM_TBSCR_REFAE))
    return CYGNUM_HAL_INTERRUPT_SIU_TB_A;

  if((tbscr & CYGARC_REG_IMM_TBSCR_REFB) && (tbscr & CYGARC_REG_IMM_TBSCR_REFBE))
    return CYGNUM_HAL_INTERRUPT_SIU_TB_B;

  return 0;
}

// enableInterruptingDevice
void TimeBase_isrDispatcher::enableInterruptingDevice(void)
{
  cyg_uint16 tbscr;

  HAL_READ_UINT16(CYGARC_REG_IMM_TBSCR, tbscr);
  tbscr |= (CYGARC_REG_IMM_TBSCR_TBE);
  tbscr &= ~(CYGARC_REG_IMM_TBSCR_REFA); // Prevent from clearing interrupts flags
  tbscr &= ~(CYGARC_REG_IMM_TBSCR_REFB); // accidently. Just do what is asked.
  HAL_WRITE_UINT16(CYGARC_REG_IMM_TBSCR, tbscr);
}

// disableInterruptingDevice
void TimeBase_isrDispatcher::disableInterruptingDevice(void)
{
  cyg_uint16 tbscr;

  HAL_READ_UINT16(CYGARC_REG_IMM_TBSCR, tbscr);
  tbscr &= ~((cyg_uint16)CYGARC_REG_IMM_TBSCR_TBE);
  tbscr &= ~(CYGARC_REG_IMM_TBSCR_REFA); // Prevent from clearing interrupts flags
  tbscr &= ~(CYGARC_REG_IMM_TBSCR_REFB); // accidently. Just do what is asked.
  HAL_WRITE_UINT16(CYGARC_REG_IMM_TBSCR, tbscr);
}
#endif // if !defined

#endif // TB_ISRDISPATCH_CPP
