#ifndef CMSEVENT_H
#define CMSEVENT_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/io/cmsobj.h"

//-------------------
// Class declarations
//-------------------
class CMS_Event
{
  public:
    typedef enum {
      Uncontrolled, Controlled, Stored } CMS_EventClass;

    CMS_EventClass       getClass(void)
                         {
                           return m_class;
                         }

  protected:
    // Prevent from instantiating the base class directly
                         CMS_Event(CMS_EventClass a_class,
                                   unsigned char a_length);
    virtual              ~CMS_Event();

    CMS_EventClass       m_class;
    unsigned char        m_length;
};

class CMS_ClientEvent : public CMS_Event,
                        public CMS_ClientObject
{
  protected:
                         CMS_ClientEvent(CMS_Event::CMS_EventClass a_class,
                                         unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                         can_io_handle_t a_handle,
                                         unsigned char a_length,
                                         bool a_numeric = true);
    virtual              ~CMS_ClientEvent();

  public:
    // Overloaded virtuals from base class CMSObject
    virtual void         dataIndication(unsigned char a_data[8],
                                        unsigned char a_length,
                                        CommunicationObject * a_cob);

  protected:
    virtual void         eventDataIndication(unsigned char * a_data) = 0;

    unsigned char        m_data[8];
    KCondition           m_condition;
    bool                 m_waiting;
};

class CMS_ServerEvent : public CMS_Event,
                        public CMS_ServerObject
{
  protected:
                         CMS_ServerEvent(CMS_Event::CMS_EventClass a_class,
                                         unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                         can_io_handle_t a_handle,
                                         unsigned char a_length,
                                         bool a_numeric = true);
    virtual              ~CMS_ServerEvent();

    void                 request(unsigned char * a_data);
    virtual void         indication(unsigned char * a_data) = 0;

  public:
    // Overloaded virtuals from base class CMSObject
    virtual void         dataIndication(unsigned char a_data[8],
                                        unsigned char a_length,
                                        CommunicationObject * a_cob);
};

//-----------------------
// The uncontrolled event
//-----------------------
// The server
class UncontrolledServerEvent : public CMS_ServerEvent
{
  protected:
    // Overloaded virtuals from base class CMS_ServerEvent
    virtual void         indication(unsigned char * a_data);
    
  public:
                         UncontrolledServerEvent(unsigned int a_inhibittime,
                                                 unsigned char a_priority,
                                                 const char * a_name,
                                                 CAN_msgid a_txcobid,
                                                 can_io_handle_t a_handle,
                                                 unsigned char a_length,
                                                 bool a_numeric = true);
    virtual              ~UncontrolledServerEvent();

    // Public members
    CMS_Error            notifyEvent(unsigned char * a_eventdata);
};

// The client
class UncontrolledClientEvent : public CMS_ClientEvent
{
  public:
                         UncontrolledClientEvent(unsigned int a_inhibittime,
                                                 unsigned char a_priority,
                                                 const char * a_name,
                                                 CAN_msgid a_rxcobid,
                                                 can_io_handle_t a_handle,
                                                 unsigned char a_length,
                                                 bool a_numeric = true);
    virtual              ~UncontrolledClientEvent();
};

//---------------------
// The controlled event
//---------------------
// The server
class ControlledServerEvent : public CMS_ServerEvent
{
  public:
    typedef enum {
      Enabled, Disabled } ControlState;

  protected:
    // Overloaded virtuals from base class CMS_ServerEvent
    virtual void         indication(unsigned char * a_data);
    
    // "To be" overloaded functions in the application
    virtual bool         setControlState(ControlState a_state, 
                                         unsigned char * a_response) = 0;

    ControlState         m_controlstate;

  public:
                         ControlledServerEvent(unsigned int a_inhibittime,
                                               unsigned char a_priority,
                                               const char * a_name,
                                               CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                               can_io_handle_t a_handle,
                                               unsigned char a_length,
                                               bool a_numeric = true);
    virtual              ~ControlledServerEvent();
                          
    // Public members
    CMS_Error            notifyEvent(unsigned char * a_eventdata);
    ControlState         getControlState(void)
                         {
                         return m_controlstate;
                         }
};

// The client
class ControlledClientEvent : public CMS_ClientEvent
{
  public:
                         ControlledClientEvent(unsigned int a_inhibittime,
                                               unsigned char a_priority,
                                               const char * a_name,
                                               CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                               can_io_handle_t a_handle,
                                               unsigned char a_length,
                                               bool a_numeric = true);
    virtual              ~ControlledClientEvent();

    // Public members
    CMS_Error            setEventControlState(bool a_enable, 
                                              unsigned char * a_response, 
                                              unsigned int a_timeout = 0);

  private:
    void                 setEventControlStateRequest(bool a_enable);
    CMS_Error            setEventControlStateConfirmation(unsigned char * a_response);
};

//-----------------
// The stored event
//-----------------
// The server
class StoredServerEvent : public CMS_ServerEvent
{
  protected:
    // overloaded virtuals from base class CMS_ServerEvent
    virtual void         indication(unsigned char * a_data);   

  public:
                         StoredServerEvent(unsigned int a_inhibittime,
                                           unsigned char a_priority,
                                           const char * a_name,
                                           CAN_msgid a_txcobid,
                                           can_io_handle_t a_handle,
                                           unsigned char a_length,
                                           bool a_numeric = true);
    virtual              ~StoredServerEvent();
    CMS_Error            storeEvent(unsigned char * a_eventdata,
                                    bool a_immediatenotify);
};

// The client
class StoredClientEvent : public CMS_ClientEvent
{
  public:
                         StoredClientEvent(unsigned int a_inhibittime,
                                           unsigned char a_priority,
                                           const char * a_name,
                                           CAN_msgid a_rxcobid,
                                           can_io_handle_t a_handle,
                                           unsigned char a_length,
                                           bool a_numeric = true);
    virtual              ~StoredClientEvent();

    CMS_Error            readEvent(unsigned char * a_eventdata, unsigned int a_timeout = 0);

  private:
    void                 requestEvent(void);
    CMS_Error            responseEvent(unsigned char * a_data);
};

#endif // CMSEVENT_H
