#ifndef CMSVAR_H
#define CMSVAR_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/io/cmsobj.h"

//-------------------
// Class declarations
//-------------------
class CMS_Variable
{
  public:
    typedef enum {
      Read_Only, Write_Only, Read_Write } CMS_VariableAccessType;
    typedef enum {
      Basic, Multiplexed } CMS_VariableClass;
 
    CMS_VariableAccessType getAccessType(void)
                           {
                             return m_accesstype;
                           }
    CMS_VariableClass      getClass(void)
                           {
                             return m_class;
                           }
  protected:
                           CMS_Variable(CMS_VariableAccessType a_accesstype,
                                        CMS_VariableClass a_class,
                                        unsigned char a_length);
    virtual                ~CMS_Variable();  

    CMS_VariableAccessType m_accesstype;
    CMS_VariableClass      m_class;
    unsigned char          m_length;
};

class CMS_ServerVariable : public CMS_Variable,
                           public CMS_ServerObject
{
  protected:
                           CMS_ServerVariable(CMS_VariableAccessType a_accesstype,
                                              CMS_VariableClass a_class,
                                              unsigned int a_inhibittime,
                                              unsigned char a_priority,
                                              const char * a_name,
                                              unsigned char a_length,
                                              bool a_numeric = true);
    virtual                ~CMS_ServerVariable();

    virtual void           readIndication(unsigned char * a_buffer, 
                                          unsigned char a_mux = 0) = 0;
    virtual bool           readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux = 0) = 0;
    virtual void           writeIndication(unsigned char * a_buffer,
                                           unsigned char a_mux = 0) = 0;
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux) = 0;
    virtual void           updateData(const unsigned char * a_data) = 0;

    void                   update(const unsigned char * a_data);

  public:
    virtual void           dataIndication(unsigned char a_data[8],
                                          unsigned char a_length,
                                          CommunicationObject * a_cob);
};

class CMS_ClientVariable : public CMS_Variable,
                           public CMS_ClientObject
{
  protected:
                           CMS_ClientVariable(CMS_VariableAccessType a_accesstype,
                                              CMS_VariableClass a_class,
                                              unsigned int a_inhibittime,
                                              unsigned char a_priority,
                                              const char * a_name,
                                              unsigned char a_length,
                                              bool a_numeric = true);
    virtual                ~CMS_ClientVariable();

    virtual void           readRequest(unsigned char a_mux = 0) = 0;
    virtual CMS_Error      readConfirmation(unsigned char *  a_rdata, 
                                            unsigned char a_mux = 0) = 0;
    virtual void           writeRequest(const unsigned char * a_wdata,
                                        unsigned char a_mux = 0) = 0;
    virtual CMS_Error      writeConfirmation(unsigned char * a_aresponse, 
                                             unsigned char a_mux = 0) = 0;

    CMS_Error              read(unsigned char * a_rdata,
                                unsigned char a_mux = 0, 
                                unsigned int a_timeout = 0);
    CMS_Error              write(unsigned char * a_wdata,
                                 unsigned char a_mux = 0,
                                 unsigned int a_timeout = 0);

    KMutex                 m_mux;
    KCondition             m_cond;
    unsigned char          m_data[8];

  public:
    virtual void           dataIndication(unsigned char a_data[8],
                                          unsigned char a_length,
                                          CommunicationObject * a_cob);
};

class CMS_BasicServerVariable : public CMS_ServerVariable
{
  protected:
                           CMS_BasicServerVariable(CMS_VariableAccessType a_accesstype,
                                                   unsigned int a_inhibittime,
                                                   unsigned char a_priority,
                                                   const char * a_name,
                                                   unsigned char a_length,
                                                   CAN_msgid a_txcobid,
                                                   CAN_msgid a_rxcobid,
                                                   can_io_handle_t a_iohandle,
                                                   bool a_numeric = true);
    virtual                ~CMS_BasicServerVariable();

    virtual void           readIndication(unsigned char * a_buffer,
                                          unsigned char a_mux = 0);
    virtual bool           readResponse(unsigned char * a_buffer,
                                        unsigned char a_mux = 0) = 0;  // Keep it abstract. This is application
                                                                       // dependant
    virtual void           writeIndication(unsigned char * a_buffer,
                                           unsigned char a_mux = 0);
    virtual bool           writeResponse(unsigned char * a_buffer,
                                         unsigned char a_mux = 0) = 0; // Keep it abstract. This is application
                                                                       // dependant
    virtual void           updateData(const unsigned char * a_data);
};

class CMS_BasicClientVariable : public CMS_ClientVariable
{
  protected:
                           CMS_BasicClientVariable(CMS_VariableAccessType a_accesstype,
                                                   unsigned int a_inhibittime,
                                                   unsigned char a_priority,
                                                   const char * a_name,
                                                   unsigned char a_length,
                                                   CAN_msgid a_txcobid,
                                                   CAN_msgid a_rxcobid,
                                                   can_io_handle_t a_iohandle,
                                                   bool a_numeric = true);
    virtual                ~CMS_BasicClientVariable();

    virtual void           readRequest(unsigned char a_mux = 0);
    virtual CMS_Error      readConfirmation(unsigned char *  a_rdata, 
                                            unsigned char a_mux = 0);
    virtual void           writeRequest(const unsigned char * a_wdata,
                                        unsigned char a_mux = 0);
    virtual CMS_Error      writeConfirmation(unsigned char * a_response, 
                                             unsigned char a_mux = 0);
};

class CMS_MultiplexedServerVariable : public CMS_ServerVariable
{
  protected:
                           CMS_MultiplexedServerVariable(CMS_VariableAccessType a_accesstype,
                                                         unsigned int a_inhibittime,
                                                         unsigned char a_priority,
                                                         const char * a_name,
                                                         unsigned char a_length,
                                                         CAN_msgid a_txcobid,
                                                         CAN_msgid a_rxcobid,
                                                         can_io_handle_t a_io_handle,
                                                         bool a_numeric = true);
    virtual                ~CMS_MultiplexedServerVariable();

    virtual void           readIndication(unsigned char * a_buffer,
                                          unsigned char a_mux);
    virtual bool           readResponse(unsigned char * a_buffer,
                                        unsigned char a_mux) = 0;      // Keep it abstract. This is application
                                                                       // dependant
    virtual void           writeIndication(unsigned char * a_buffer,
                                           unsigned char a_mux);
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux) = 0;     // Keep it abstract. This is application
                                                                       // dependant
    virtual void           updateData(const unsigned char * a_data) {} // Prevent class from being abstract. This 
                                                                       // will never show in the public interface
};

class CMS_MultiplexedClientVariable : public CMS_ClientVariable
{
  protected:
                           CMS_MultiplexedClientVariable(CMS_VariableAccessType a_accesstype,
                                                         unsigned int a_inhibittime,
                                                         unsigned char a_priority,
                                                         const char * a_name,
                                                         unsigned char a_length,
                                                         CAN_msgid a_txcobid,
                                                         CAN_msgid a_rxcobid,
                                                         can_io_handle_t a_io_handle,
                                                         bool a_numeric = true);
    virtual                ~CMS_MultiplexedClientVariable();

    virtual void           readRequest(unsigned char a_mux);
    virtual CMS_Error      readConfirmation(unsigned char *  a_rdata, 
                                            unsigned char a_mux);
    virtual void           writeRequest(const unsigned char * a_wdata, 
                                        unsigned char a_mux);
    virtual CMS_Error      writeConfirmation(unsigned char * a_response, 
                                             unsigned char a_mux);
};

template <class Type>
class ROBasicClientVar : public CMS_BasicClientVariable
{
  public:
                           ROBasicClientVar(unsigned int a_inhibittime, 
                                            unsigned char a_priority, 
                                            const char * a_name, 
                                            CAN_msgid a_rxcobid, 
                                            can_io_handle_t a_handle,
                                            bool a_numeric = true);
    virtual                ~ROBasicClientVar() {}

    CMS_Error              readVariable(Type * a_data, 
                                        unsigned char a_response[8], 
                                        unsigned int a_timeout = 0);
};

template <class Type>
class RWBasicClientVar : public CMS_BasicClientVariable
{
  public:
                           RWBasicClientVar(unsigned int a_inhibittime,
                                            unsigned char a_priority,
                                            const char * a_name,
                                            CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                            can_io_handle_t a_handle,
                                            bool a_numeric = true);
    virtual                ~RWBasicClientVar() {}

    CMS_Error              readVariable(Type * a_data, 
                                        unsigned char a_response[8], 
                                        unsigned int a_timeout = 0);
    CMS_Error              writeVariable(Type a_data, 
                                         unsigned char a_response[8], 
                                         unsigned int a_timeout = 0);
};

template <class Type>
class WOBasicClientVar : public CMS_BasicClientVariable
{
  public:
                           WOBasicClientVar(unsigned int a_inhibittime,
                                            unsigned char a_priority,
                                            const char * a_name,
                                            CAN_msgid a_txcobid,
                                            can_io_handle_t a_handle,
                                            bool a_numeric = true);
    virtual                ~WOBasicClientVar() {}

    CMS_Error              writeVariable(Type a_data, 
                                         unsigned char a_response[8], 
                                         unsigned int a_timeout = 0);
};

template <class Type>  
class ROBasicServerVar : public CMS_BasicServerVariable
{
  public:
                           ROBasicServerVar(unsigned int a_inhibittime,
                                            unsigned char a_priority,
                                            const char * a_name,
                                            CAN_msgid a_txcobid,
                                            can_io_handle_t a_handle,
                                            bool a_numeric = true);
    virtual                ~ROBasicServerVar() {}

    void                   updateVariable(Type a_data);

  protected:
    virtual bool           readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux = 0);
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux = 0);
};

template <class Type> 
class RWBasicServerVar : public CMS_BasicServerVariable
{
  public:
                           RWBasicServerVar(unsigned int a_inhibittime,
                                            unsigned char a_priority,
                                            const char * a_name,
                                            CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                            can_io_handle_t a_handle,
                                            bool a_numeric = true);
    virtual                ~RWBasicServerVar() {}

  protected:
    virtual bool           dataRequest(Type * a_data, 
                                       unsigned char * a_response) = 0;
    virtual bool           dataIndication(Type a_data, 
                                          unsigned char * a_response) = 0;

    virtual bool           readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux = 0);
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux = 0);
};

template <class Type>
class WOBasicServerVar : public CMS_BasicServerVariable
{
  public:
                           WOBasicServerVar(unsigned int a_inhibittime,
                                            unsigned char a_priority,
                                            const char * a_name,
                                            CAN_msgid a_rxcobid,
                                            can_io_handle_t a_handle,
                                            bool a_numeric = true);
    virtual                ~WOBasicServerVar() {}

  protected:
    virtual void           dataIndication(Type a_data) = 0;

    virtual bool           readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux = 0);
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux = 0);
};


template <class Type>
class ROMuxClientVar : public CMS_MultiplexedClientVariable
{
  public:
                           ROMuxClientVar(unsigned int a_inhibittime,
                                          unsigned char a_priority,
                                          const char * a_name,
                                          CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                          can_io_handle_t a_handle,
                                          bool a_numeric = true);
    virtual                ~ROMuxClientVar() {}

    CMS_Error              readVariable(Type * a_data, 
                                        unsigned char a_mux, 
                                        unsigned char a_response[8], 
                                        unsigned int a_timeout);
};

template <class Type>
class ROMuxServerVar : public CMS_MultiplexedServerVariable
{
  public:
                           ROMuxServerVar(unsigned int a_inhibittime,
                                          unsigned char a_priority,
                                          const char * a_name,
                                          CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                          can_io_handle_t a_handle,
                                          bool a_numeric = true);
    virtual                ~ROMuxServerVar() {}

  protected:
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux);
    virtual bool           readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux);

    virtual bool           dataRequest(Type * a_data, 
                                       unsigned char a_mux, 
                                       unsigned char * a_response) = 0;
};

template <class Type>
class WOMuxClientVar : public CMS_MultiplexedClientVariable
{
  public:
                           WOMuxClientVar(unsigned int a_inhibittime,
                                          unsigned char a_priority,
                                          const char * a_name,
                                          CAN_msgid a_cobid,
                                          can_io_handle_t a_handle,
                                          bool a_numeric = true);
    virtual                ~WOMuxClientVar() {}

    CMS_Error              writeVariable(Type a_data, 
                                         unsigned char a_mux, 
                                         unsigned char a_response[8], 
                                         unsigned int a_timeout = 0);
};

template <class Type>
class WOMuxServerVar : public CMS_MultiplexedServerVariable
{
  public:
                           WOMuxServerVar(unsigned int a_inhibittime,
                                          unsigned char a_priority,
                                          const char * a_name,
                                          CAN_msgid a_cobid,
                                          can_io_handle_t a_handle,
                                          bool a_numeric = true);
    virtual                ~WOMuxServerVar() {}

  protected:
    virtual void           dataIndication(Type a_data, unsigned char a_mux) = 0;

    virtual bool           readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux);
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux);
};

template <class Type>
class RWMuxClientVar : public CMS_MultiplexedClientVariable
{
  public:
                           RWMuxClientVar(unsigned int a_inhibittime,
                                          unsigned char a_priority,
                                          const char * a_name,
                                          CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                          can_io_handle_t a_handle,
                                          bool a_numeric = true);
    virtual                ~RWMuxClientVar() {}

    CMS_Error              readVariable(Type * a_data, 
                                        unsigned char a_mux, 
                                        unsigned char a_response[8], 
                                        unsigned int a_timeout = 0);
    CMS_Error              writeVariable(Type a_data,
                                         unsigned char a_mux,
                                         unsigned char a_response[8],
                                         unsigned int a_timeout = 0);
};

template <class Type>
class RWMuxServerVar : public CMS_MultiplexedServerVariable
{
  public:
                           RWMuxServerVar(unsigned int a_inhibittime,
                                          unsigned char a_priority,
                                          const char * a_name,
                                          CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                          can_io_handle_t a_handle,
                                          bool a_numeric = true);
    virtual                ~RWMuxServerVar() {}

  protected:
    virtual bool           dataIndication(Type a_data, 
                                          unsigned char a_mux, 
                                          unsigned char * a_response) = 0;
    virtual bool           dataRequest(Type * a_data, 
                                       unsigned char a_mux, 
                                       unsigned char * a_response) = 0;

    virtual bool           readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux);
    virtual bool           writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux);
};

//--------------------------------
// Include the implementation file
//--------------------------------
#include "cyg/io/cmsvar.inl"

#endif // CMSVAR_H
