#ifndef CANMESSAGEDISPATCHER_CXX
#define CANMESSAGEDISPATCHER_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/io/canmsgdisp.h"
#include "cyg/io/comobj.h"
#include <cyg/kal/kdefs.h>

//---------------------
// Class implementation
//---------------------
// Constructor
CanMessageDispatcher::CanMessageDispatcher(char * a_name)
    : KThread(CYGNUM_CAL_THREAD_PRIORITY, a_name, &m_stack[0], CYGNUM_CAL_THREAD_STACK_SIZE),
      m_io_handle(0)
{ // Initialise the COB matrix to all zeros
  unsigned short i,j;
  for(i = 0; i < 256; i++)
  {
    for(j = 0; j < 8; j++)
      m_cob[i][j] = 0;
  }
}

// Provide an (empty) virtual destructor
CanMessageDispatcher::~CanMessageDispatcher()
{
}

// attach
void CanMessageDispatcher::attach(CommunicationObject * a_cob)
{
  volatile cyg_uint16 cob_id = a_cob->getCOBid();
  volatile unsigned char b_index = (unsigned char)((cob_id & 0x7F8) >> 3);
  volatile unsigned char s_index = (unsigned char)(cob_id & 0x07);

  m_cob[b_index][s_index] = a_cob;
}

// detach
void CanMessageDispatcher::detach(CommunicationObject * a_cob)
{
  volatile cyg_uint16 cob_id = a_cob->getCOBid();
  volatile unsigned char b_index = (unsigned char)((cob_id & 0x7F8) >> 3);
  volatile unsigned char s_index = (unsigned char)(cob_id & 0x07);

  m_cob[b_index][s_index] = 0;
}

// start
void CanMessageDispatcher::start(can_io_handle_t a_handle)
{
  if(!m_io_handle)
  {
    m_io_handle = a_handle;
    resume();
  }
}

// stop
void CanMessageDispatcher::stop(void)
{
  if(m_io_handle)
  {
    suspend();
    m_io_handle = 0;
  }
}

//----------------
// Private members
//----------------
// messageLoop
void CanMessageDispatcher::threadEntry(void)
{
  can_message msg;
  DWORD len;
  for(;;)
  {
    len = sizeof(can_message);
    if(can_io_read(m_io_handle, &msg, &len) == ENOERR)
    {
      volatile unsigned char b_index = (unsigned char)((msg.m_CAN_id & 0x7F8) >> 3);
      volatile unsigned char s_index = (unsigned char)(msg.m_CAN_id & 0x07);

      _ASSERT(m_cob[b_index][s_index] != 0, "Unregistered COB id received");
      m_cob[b_index][s_index]->dataIndication(msg.m_Data, msg.m_DLC);
    }
    else
    {
    }
  }
}

#endif // CANMESSAGEDISPATCHER_CXX
