#ifndef CANIO_H
#define CANIO_H

#ifdef __cplusplus
extern "C" {
#endif

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <cyg/io/canconfig_keys.h>
#include <cyg/infra/cyg_type.h>

//---------------------
// Supported baud rates
//---------------------
typedef enum {
  CAN_BAUD_KBIT_1000 = 1,
  CAN_BAUD_KBIT_800,
  CAN_BAUD_KBIT_500,
  CAN_BAUD_KBIT_250,
  CAN_BAUD_KBIT_125,
  CAN_BAUD_KBIT_100,
  CAN_BAUD_KBIT_50,
  CAN_BAUD_KBIT_20,
  CAN_BAUD_KBIT_10
} can_baud_rate_t;


#define _CAN_BAUD_RATE(n) CAN_BAUD_KBIT_##n
#define CAN_BAUD_RATE(n) _CAN_BAUD_RATE(n)

//----------------------
// Electrical bus levels
//----------------------
typedef enum {
  CAN_RX_LOGICAL_ZERO_DOMINANT = 1,
  CAN_RX_LOGICAL_ONE_DOMINANT
} can_rx_mode_t;

typedef enum {
  CAN_TX_FULL_CMOS_POSITIVE_POLARITY = 1,
  CAN_TX_FULL_CMOS_NEGATIVE_POLARITY,
  CAN_TX_OPEN_DRAIN
} can_tx_mode_t;

typedef struct {
  can_baud_rate_t  m_baud;
  can_rx_mode_t    m_rx_mode;
  can_tx_mode_t    m_tx_mode;
} can_info_t;

#define CAN_INFO_INIT(_baud, _rx_mode, _tx_mode) \
  {_baud, _rx_mode, _tx_mode}

//----------------------------------------------------------------------
// Action to performed under interrupt when the indicated id is received
//----------------------------------------------------------------------
typedef struct {
  cyg_uint32 m_can_id;
  void (* m_action)(void);
} can_rxid_action_t;

#define CAN_RXID_ACTION_INIT(_can_id, _action)   \
  {_can_id, _action}

//--------------------------
// The default configuration
//--------------------------
#define CAN_BAUD_DEFAULT                     CAN_BAUD_KBIT_1000
#define CAN_RX_MODE_DEFAULT                  CAN_RX_LOGICAL_ZERO_DOMINANT
#define CAN_TX_MODE_DEFAULT                  CAN_TX_FULL_CMOS_POSITIVE_POLARITY

#ifdef __cplusplus
}
#endif

#endif // CANIO_H
