#ifndef KTIMER_H
#define KTIMER_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <cyg/kernel/kapi.h>
class KTimerBase;
class KTimer;
class KTimerList;

//-----------------
// Type definitions
//-----------------
typedef cyg_handle_t KTimerHandle;

//------------------
// Class declaration
//------------------
// Class KTimerClient
class KTimerClient
{
  public :
                          KTimerClient() {}
    virtual               ~KTimerClient();

    virtual void          timeOut(KTimer * a_timer) = 0;
};

// Class KTimer
class KTimer : public cyg_alarm
{
  friend class KTimerList;
  friend class KTimerBase;

  public:
    typedef enum {
      Foreground, Background } KTimerSchedule;

    typedef enum {
      High, Normal, Low } KTimerPrecision;

                          KTimer(KTimerClient * a_client);
                          KTimer(KTimerClient * a_client,
                                 KTimerPrecision a_precision,
                                 KTimerSchedule a_schedule);
    virtual               ~KTimer();

    void                  start(unsigned long a_microsecs);
    void                  oneShot(unsigned long a_microsecs);
    void                  stop(void);

  private:
    static void           stimeOut(KTimerHandle a_timer, void * a_data);
    void                  timeout(void);

    KTimerSchedule        m_schedule;
    KTimerPrecision       m_precision;
    KTimerClient *        m_client;
    KTimerBase *          m_base;

    // Private attributes to enable simple listing
    KTimer *              m_prev;
    KTimer *              m_next;

    static KTimerBase *   m_highbase;
    static KTimerBase *   m_normalbase;
    static KTimerBase *   m_lowbase;

    cyg_handle_t          m_handle;
};

// Class KTimerList
class KTimerList
{
  public:
                          KTimerList();
    virtual               ~KTimerList();

    void                  add(KTimer * a_timer)
                          {
                            a_timer->m_next = m_head.m_next;
                            a_timer->m_prev = &m_head;
                            (m_head.m_next)->m_prev = a_timer;
                            m_head.m_next = a_timer;
                          }
    KTimer *              remove(void)
                          {
                            KTimer * result = m_tail.m_prev;

                            if(result != &m_head)
                            {
                              m_tail.m_prev = result->m_prev;
                              (result->m_prev)->m_next = &m_tail;
                              result->m_next = 0;
                              result->m_prev = 0;

                              return result;
                            }
                            return 0;
                          }
    void                  remove(KTimer * a_timer)
                          {
                            (a_timer->m_prev)->m_next = a_timer->m_next;
                            (a_timer->m_next)->m_prev = a_timer->m_prev;
                            a_timer->m_next = 0;
                            a_timer->m_prev = 0;
                          }

  private:
    KTimer              m_head;
    KTimer              m_tail;
};

#endif // KTIMER_H
